#!/usr/bin/python2
# -*- coding: utf-8 -*-
import sys
import os
import pdb

#---------------------------these classes used to output differet kind of configure items type-------------------------
class KeyValDisplay:
    '''
    The configure item that use a [key=value] format, use this class to display
    '''
    def __init__(self, key, value):
        self.key = key
        self.value = value
        
    def __str__(self):
        return "%s=%s" % (self.key, self.value)


class ValDisplay:
    '''
    The configure items that use a [value] format, use this class to display
    '''
    def __init__(self, value):
        self.value = value
    
    def __str__(self):
        return "%s" % (self.value)
#---------------------------------------------------------------------------------------------------------------

#-------------------------FileCmper--------------------------------------------------------
class INIFileCMPer:
    '''The Compare class for [.ini] files
    '''
    # Define the state of parsing
    IDLE = 0
    INSESSION =1
    
    def __init__(self):
        # parse the configure item in the .ini files as a map
        # it shout be { session1: {key1: value1, key2, value2}, session2: {key1: value1, ke2: value2} }    
        self.dstConMap = {}
        self.defaultConMap = {}
        self.state = INIFileCMPer.IDLE
        self.curSession = ""
        
    def load(self, dstFile, defaultFile):
        ''' Load the files that needed to be compared
        and construct the Configure item map.
        '''
        self.dstFile = dstFile
        self.defaultFile = defaultFile
        
        #construct the configure items map
        if False == self.__parseDst():
            print "Parse the dest configure file: [%s] failed" \
            % (self.dstFile)
            return False
            
        if False == self.__parseDefault():
            print "Parse the default configure file: [%s] failed" \
                    % (self.defaultFile)
            return False
        
        return True
        
    def doCompare(self):
        '''The configure items in the default file but not in the dst file
        But some items in the dst file but not in the default file
        '''
        self.__doCompare(self.dstConMap, self.defaultConMap, self.defaultFile, True)
        self.__doCompare(self.defaultConMap, self.dstConMap, self.dstFile, False)

    def reset(self):
        self.dstConMap.clear()
        self.defaultConMap.clear()
        self.__innerReset()
        return
        
        
    def __doCompare(self, dstConMap, defaultConMap, defaultFile, outputDiff):
        '''Compare the dst configure items map and the default 
        configure items map and find out the different ones
        '''
        for session, items in dstConMap.items():
            if False == defaultConMap.has_key(session):
                #For session that use file name(the configure item that in no session)
                if session != os.path.basename(defaultFile).strip():
                    print "  @@DIFF: There is no session [%s] in the " \
                    "file [%s]." \
                    % (session, defaultFile)
                else:
                    print "  @DIFF: There is no below configure items in the " \
                    "file [%s]: " % (defaultFile)
                    for key, value in items.items():
                        print "    [", value[1], "]"
                continue
            
            for key, value in items.items():
                if False == defaultConMap[session].has_key(key):
                    if session != os.path.basename(defaultFile).strip():
                        print "  @@DIFF: The configure item [%s] " \
                        "in session [%s] is not in the file [%s]." \
                        % (key, session, defaultFile)
                    else:
                        print "  @@DIFF: The configure item [%s] " \
                        "is not in the file [%s]." \
                        % (key, defaultFile)
                    continue
                
                elif True == outputDiff and \
                        value[0] != defaultConMap[session][key][0]:
                    print "  @@DIFF: The value of configure item [%s] " \
                    "in session [%s] is defferent from the default. " \
                    "It is [%s], and the default is [%s]. " \
                    % (key, session, value[1], \
                        defaultConMap[session][key][1])
        return
        
    def __innerReset(self):
        self.state = INIFileCMPer.IDLE
        self.curSession = ""

    def __parseDst(self):
        self.__parse(self.dstFile, self.dstConMap)
        
    def __parseDefault(self):
        self.__parse(self.defaultFile, self.defaultConMap)
    
    def __parse(self, file, ConMap):
        #remove the BOM characters in the file head
        fp = open(file, 'r')
        if "\xef\xbb\xbf" == fp.read(3):
            fp.seek(3)
        else:
            fp.seek(0)
            
        for line in fp.readlines():
            line = line.strip()
            
            # Discard the empty line and the comments
            if not len(line) or line.startswith('#') or line.startswith(';'):
                continue
            
            # Get the session name
            elif line.startswith('[') and line.endswith(']'):
                self.state = INIFileCMPer.INSESSION
                self.curSession = line[ 1:len(line)-1 ].strip()
                ConMap[self.curSession] = {}
                
            elif self.state == INIFileCMPer.INSESSION:
                #for those items that not use the format [key: value] 
                if '=' not in line:
                    ConMap[self.curSession][line] = ('', ValDisplay(line))
                else:
                    Partition = line.split('=', 1)
                    key = Partition[0].strip()
                    value = ''
                    if len(Partition) == 2:
                        value = Partition[1].strip()
                    ConMap[self.curSession][key] \
                            = (value, KeyValDisplay(key, value))

            elif self.state == INIFileCMPer.IDLE and \
                not (line.startswith('[') and line.endswith(']') ):
                #for those items that not belong to any session, use the file name as session
                self.curSession = os.path.basename(file).strip()
                if False == ConMap.has_key(self.curSession):
                    ConMap[self.curSession] = {}
                #some configure item is define as the format "key=value", but some is just a "value"
                if "=" in line:
                    Partition = line.split('=', 1)
                    key = Partition[0].strip()
                    value = ''
                    if len(Partition) == 2:
                        value = Partition[1].strip()
                    ConMap[self.curSession][key] \
                            = (value, KeyValDisplay(key, value))
                else:
                    ConMap[self.curSession][line] = ('', ValDisplay(line))
            
            else:
                print "The line [%s] is not processed." % (line)
        
        fp.close()
        self.__innerReset()
        return True
#----------------------------------------------------------------------------------------------

#-----------------Configure compare manager-------------------------------------------
class ConCmpMgr:
    '''Control class that manager the compre process
    '''
    #define the mapping between the file type and the compare class
    FileTypeCmpMap = {'.ini': INIFileCMPer}
    
    def __init__(self, filePath, defaultFilePath, fileType):
        self.filePath = filePath
        self.defaultFilePath = defaultFilePath
        self.fileType = fileType
        self.fileList = []
        self.defaultFileList = []
        
    def initialize(self):
        '''get all the files of the sepcial type 
        in the path and the default path
        '''
        if False == os.path.exists(self.filePath):
            print "File path [%s] is not exists." % (self.filePath)
            return False
        if False == os.path.exists(self.defaultFilePath):
            print "Default file path [%s] is not exists." \
            % (self.defaultFilePath)
            return False

        self.__getFileList(self.filePath, self.fileType, self.fileList)
        self.__getFileList(self.defaultFilePath, self.fileType, \
                            self.defaultFileList)
        
        if len(self.fileList) > 0 and len(self.defaultFileList) > 0 :
            self.__setCMPer(self.fileType)
            return True
        else:
            print "Initialize ConCmpMgr for file type: [%s] failed."\
            % (self.fileType)
            return False
        
        return Ture
            
    def compareFiles(self):
        '''Compare the files in the fileList one by one
        '''
        for file in self.fileList:
            print "COMPARE FILE: [%s] " % (os.path.join(self.filePath,file))
            if file not in self.defaultFileList:
                print "  @@DIFF: There is no file [%s] " \
                "in the default path [%s]" \
                % (file, self.defaultFilePath)
                continue
            
            if False == self.cmper.load(os.path.join(self.filePath,file), \
                                os.path.join(self.defaultFilePath, file) ):
                return
                
            self.cmper.doCompare()
            self.cmper.reset()

        return
            
    def __getFileList(self, filePath, fileType, fileList):
        '''
        #...........
        #>>>FindPath is the input dir that would be searched
        #>>>FlagStr='.ini' || '.txt' || other kind
        #>>>FileList=GetFileList(FindPath,FlagStr) #
        '''
        fileNames = os.listdir(filePath)
        if len(fileNames)>0:
            for fn in fileNames:
                if len(fileType)>0:
                    if fn.endswith(fileType):
                        fileList.append(fn)
                else:
                    fileList.append(fn)
        if len(fileList)>0:
            fileList.sort()
        else:
            print "In file path %s, there is no file type: %s" \
            % (filePath, fileType)

    def __setCMPer(self, fileType):
        self.cmper = ConCmpMgr.FileTypeCmpMap[fileType]()
        
#---------------------------------------------------------------------------------------------------------------------



#--------------main program------------------------------------------------------
CONFIG_FILE_PATH = "/etc/iscan"
# The default configure file was putted in the dir [/usr/iwss/migration/migration_packages] as .tar.gz package 
# in the run ENV
RUNDEFAULT_CONFIG_FILE_PATH = "/usr/iwss/migration/migration_packages"
# Make a temp dir to put the default configure files
TMP_DEFAULT_CONFIG_PATH = "/var/iwss/default/configrueCompare"
#Make a tem dir to put the modified configure files
TMP_MODIFIED_CONFIG_PATH = "/var/iwss/modified/configrueCompare"

def getIniRunConfig():
    '''Started by the CDT script
    '''
    if (bStartFromCDT):
        return CONFIG_FILE_PATH

    '''Get the configure file path that has been modified by the customer
    '''
    strModifiedConfig = raw_input("Please input the path and name of zip "\
    "configure file that GET FROM CDT.\nIf you have import the configure "\
    "to the current ENV, just left it empty: ")
    
    if ("" == strModifiedConfig):
        return CONFIG_FILE_PATH
        
    if False == os.path.exists(TMP_MODIFIED_CONFIG_PATH):
        print "Create temp path [%s] failed." % TMP_MODIFIED_CONFIG_PATH
        sys.exit(101)

    os.popen("tar xf %s -C %s" % (strModifiedConfig, TMP_MODIFIED_CONFIG_PATH))
    return os.path.join(TMP_MODIFIED_CONFIG_PATH, "Configurations")
    
def getIniDefaultConfig():
    '''Started by the CDT script
    '''
    defaultZipFile = ""
    
    if (not bStartFromCDT):  
        '''Get the default configure file path on the running Env
        '''
        defaultZipFile = raw_input("Please input the path and name of DEFAULT "\
        "zip configure file.\nIf current ENV has installed the same version of "\
        "IWSVA as the customer, just left it empty: ")

    if "" == defaultZipFile:
        configureZip = ""
        var = os.popen("/usr/iwss/iwssd -v").readline()
        version = var[var.find("IWSVA ")+6: var.find("_Build")].strip()
        zipfileList = os.listdir(RUNDEFAULT_CONFIG_FILE_PATH)
        
        for zip in zipfileList:
            if version in zip:
                configureZip = zip
                
        if "" == configureZip:
            return ""
            
        defaultZipFile = os.path.join(RUNDEFAULT_CONFIG_FILE_PATH, configureZip)
        
    if False == os.path.exists(TMP_DEFAULT_CONFIG_PATH):
        print "Create temp path [%s] failed." % TMP_DEFAULT_CONFIG_PATH
        sys.exit(101)
    
    os.popen("tar xf %s -C %s" % (defaultZipFile, TMP_DEFAULT_CONFIG_PATH))
    return os.path.join(TMP_DEFAULT_CONFIG_PATH, "Configurations")
    
    
def makeTmpPath():
    os.popen("mkdir -p %s" % TMP_DEFAULT_CONFIG_PATH)
    os.popen("mkdir -p %s" % TMP_MODIFIED_CONFIG_PATH)
    return
    
        
def clearTmpPath():
    os.system("rm -rf %s" % TMP_DEFAULT_CONFIG_PATH)
    os.system("rm -rf %s" % TMP_MODIFIED_CONFIG_PATH)
    return

    
bStartFromCDT = False
if (len(sys.argv) > 1 and sys.argv[1] == "CDT"):
    bStartFromCDT = True
    
makeTmpPath()
    
conCmpMgr = ConCmpMgr(getIniRunConfig(), \
                      getIniDefaultConfig(), \
                      '.ini')

if False == conCmpMgr.initialize():
    sys.exit(101)
    
conCmpMgr.compareFiles()

clearTmpPath()

sys.exit(0)

    