#!/bin/sh

MODULE_NAME=DiagnosticTool

bMultiCheck=0

if [ "$1" = "--multicheck" ]; then
	bMultiCheck=1
fi

MODULE_DIR=/var/iwss/DiagnosticTool
SUMMARY_REPORT_FILE_NAME=summary_report.txt
SUMMARY_REPORT_FILE=$MODULE_DIR/$SUMMARY_REPORT_FILE_NAME
DETAIL_REPORT_FILE_NAME=detail_report.txt
DETAIL_REPORT_FILE=$MODULE_DIR/$DETAIL_REPORT_FILE_NAME
MODULE_CONFIG_FILE=$MODULE_DIR/diagnostic_tool.ini
CONCMP_REPORT_FILE_NAME=configure_compare_report.txt
CONCMP_REPORT_FILE=$MODULE_DIR/$CONCMP_REPORT_FILE_NAME


IWSS_HOME=/var/iwss
INTSCAN_CONFIG_FILE=$IWSS_HOME/intscan.ini

CURR_TIME=`date '+%Y%m%d_%H%M%S'`

# First Check for Multiple Diagnostic Tools Running
if [ -f $MODULE_DIR/$MODULE_NAME.pid ]; then
	PREVIOUS_PID=`cat $MODULE_DIR/$MODULE_NAME.pid`
fi

if [ -z "$PREVIOUS_PID" ]; then
	if [ $bMultiCheck -eq 0 ]; then
		CURR_PID=$$
		echo $CURR_PID > $MODULE_DIR/$MODULE_NAME.pid
		chmod a+rw $MODULE_DIR/$MODULE_NAME.pid &> /dev/null
	else
		echo "No other Diagnostic Tool is running"
		exit 0
	fi
else
	PREVIOUS_PIDINFO=`ps aux | grep "$PREVIOUS_PID" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_NAME"`
	if [ -z "$PREVIOUS_PIDINFO" ]; then
		# Second Check for Multiple Diagnostic Tools Running
		if [ -f $MODULE_DIR/$MODULE_NAME.pid ] ; then
			PREVIOUS_PID=`cat $MODULE_DIR/$MODULE_NAME.pid`
		fi

		if [ -z "$PREVIOUS_PID" ]; then
			if [ $bMultiCheck -eq 0 ]; then
				CURR_PID=$$
				echo $CURR_PID > $MODULE_DIR/$MODULE_NAME.pid
				chmod a+rw $MODULE_DIR/$MODULE_NAME.pid &> /dev/null
			else
				echo "No other Diagnostic Tool is running"
				exit 0
			fi
		else
			PREVIOUS_PIDINFO=`ps aux | grep "$PREVIOUS_PID" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_NAME"`

			if [ -z "$PREVIOUS_PIDINFO" ]; then
				if [ $bMultiCheck -eq 0 ]; then
					CURR_PID=$$
					echo $CURR_PID > $MODULE_DIR/$MODULE_NAME.pid
					chmod a+rw $MODULE_DIR/$MODULE_NAME.pid &> /dev/null
				else
					echo "No other Diagnostic Tool is running"
					exit 0
				fi
			else
				echo "Another Diagnostic Tool is Running"
				exit 120
			fi
		fi
	else
	 	echo "Another Diagnostic Tool is Running"
		exit 120
	fi
fi

# Check the report files
if [ -f $SUMMARY_REPORT_FILE ]; then
	rm -f $SUMMARY_REPORT_FILE
fi
touch $SUMMARY_REPORT_FILE
chmod 666 $SUMMARY_REPORT_FILE

if [ -f $DETAIL_REPORT_FILE ]; then
	rm -f $DETAIL_REPORT_FILE
fi
touch $DETAIL_REPORT_FILE
chmod 666 $DETAIL_REPORT_FILE

if [ -f $CONCMP_REPORT_FILE ]; then
	rm -f $CONCMP_REPORT_FILE
fi
touch $CONCMP_REPORT_FILE
chmod 666 $CONCMP_REPORT_FILE

# Start to diagnose
echo "================================ Diagnostic Tool Start ================================" >> $DETAIL_REPORT_FILE
echo `date` >> $DETAIL_REPORT_FILE

# Check the config file of this tool
if [ ! -f $MODULE_CONFIG_FILE ]; then
	echo "Can not find file $MODULE_CONFIG_FILE" >> $DETAIL_REPORT_FILE
	echo "================================ Diagnostic Tool Failed ================================" >> $DETAIL_REPORT_FILE
	exit 100
fi

echo "[Common]" >> $SUMMARY_REPORT_FILE
diagnosticTime=`date "+%Y-%m-%d %H:%M:%S"`
echo -e "DiagnosticTime=$diagnosticTime\n" >> $SUMMARY_REPORT_FILE
# Parse the config file
echo "Start to parse configuration file" >> $DETAIL_REPORT_FILE
sections=`cat $MODULE_CONFIG_FILE | sed 's/^[ \t]*//g' | sed 's/[ \t]*$//g' | grep -e '^\[.*\]' | grep -v '\[Common\]'`
for section in $sections
do
	echo $section >> $DETAIL_REPORT_FILE
	module=`echo ${section:1:${#section}-2}`
	#echo $module
	name=`awk -F '=' '/\['"${module}"'\]/{a=1}a==1&&$1~/^ModuleName/{print $2;exit}' $MODULE_CONFIG_FILE`
	if [ -z "$name" ]; then
		echo "Get ModuleName Failed" >> $DETAIL_REPORT_FILE
		exit 100
	fi
	echo "ModuleName=$name" >> $DETAIL_REPORT_FILE
	description=`awk -F '=' '/\['"${module}"'\]/{a=1}a==1&&$1~/^Description/{print $2;exit}' $MODULE_CONFIG_FILE`
	if [ -z "$description" ]; then
		echo "Get Description Failed" >> $DETAIL_REPORT_FILE
		exit 100
	fi
	echo -e "Description=$description" >> $DETAIL_REPORT_FILE
	program=`awk -F '=' '/\['"${module}"'\]/{a=1}a==1&&$1~/^ProgramPath/{print $2;exit}' $MODULE_CONFIG_FILE`
	if [ -z "$program" ]; then
		echo "Get ProgramPath Failed" >> $DETAIL_REPORT_FILE
		exit 100
	fi
	echo "ProgramPath=$program" >> $DETAIL_REPORT_FILE
	display=`awk -F '=' '/\['"${module}"'\]/{a=1}a==1&&$1~/^ReportDisplay/{print $2;exit}' $MODULE_CONFIG_FILE`
	if [ -z "$display" ]; then
		echo "Get ReportDisplay Failed" >> $DETAIL_REPORT_FILE
		exit 100
	fi
	echo "ReportDisplay=$display" >> $DETAIL_REPORT_FILE

	# Run sub-tool
	echo "**************************************************************************************" >> $DETAIL_REPORT_FILE
	echo "     $name" >> $DETAIL_REPORT_FILE
	echo "**************************************************************************************" >> $DETAIL_REPORT_FILE
	echo "$section" >> $SUMMARY_REPORT_FILE
	echo "ReportDisplay=$display" >> $SUMMARY_REPORT_FILE

	chmod a+x $program &>> $DETAIL_REPORT_FILE
    if [ "$section" = "[ConfigureCompare]" ]; then
        echo "The comparing result detail please check the file [configure_compare_report.txt]" >> $DETAIL_REPORT_FILE
        $program "CDT" &>> $CONCMP_REPORT_FILE
    else
        $program &>> $DETAIL_REPORT_FILE
    fi
	case $? in
	0)
		printf "%-30s%s\n" "$display" Success
		echo -e "Result=Success\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" Success >> $DETAIL_REPORT_FILE
		;;
	101)
		printf "%-30s%s\n" "$display" Fail
		echo -e "Result=Fail\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" Fail >> $DETAIL_REPORT_FILE
		;;
	102)
		printf "%-30s%s\n" "$display" NeedCheck
		echo -e "Result=NeedCheck\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" NeedCheck >> $DETAIL_REPORT_FILE
		;;
	103)
		printf "%-30s%s\n" "$display" NoConfigure
		echo -e "Result=NoConfigure\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" NoConfigure >> $DETAIL_REPORT_FILE
		;;
	104)
		printf "%-30s%s\n" "$display" Warning
		echo -e "Result=Warning\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" Warning >> $DETAIL_REPORT_FILE
		;;
	*)
		printf "%-30s%s\n" "$display" Error
		echo -e "Result=Error\n" >> $SUMMARY_REPORT_FILE
		printf "%-30s%s\n" "$display" Error >> $DETAIL_REPORT_FILE
		;;
	esac
	echo -e "**************************************************************************************\n" >> $DETAIL_REPORT_FILE
done

echo "================================ Diagnostic Tool End =================================" >> $DETAIL_REPORT_FILE

# Compress and archive files
echo "Compress and archive files" >> $DETAIL_REPORT_FILE
report_dir=`awk -F '=' '/\[Common\]/{a=1}a==1&&$1~/^ReportPath/{print $2;exit}' $MODULE_CONFIG_FILE`
if [ -z "$report_dir" ]; then
	echo "Get ReportPath Failed" >> $DETAIL_REPORT_FILE
	echo "================================ Diagnostic Tool Failed ================================" >> $DETAIL_REPORT_FILE
	exit 100
fi

if [ ! -d "$report_dir" ]; then
	mkdir -p $report_dir &>> $DETAIL_REPORT_FILE
fi

cd $report_dir &>> $DETAIL_REPORT_FILE
cp -f $SUMMARY_REPORT_FILE $DETAIL_REPORT_FILE $CONCMP_REPORT_FILE ./
sed -i'' "s/$/\r/" $DETAIL_REPORT_FILE_NAME
target_file=Report_$CURR_TIME.tar.gz
tar zcvf $target_file $SUMMARY_REPORT_FILE_NAME $DETAIL_REPORT_FILE_NAME $CONCMP_REPORT_FILE_NAME &>> /dev/null
if [ $? -ne 0 ]; then
	echo "Compress and archive files Failed" >> $DETAIL_REPORT_FILE
	exit 100
fi
chmod a+w $target_file
rm -f $SUMMARY_REPORT_FILE_NAME $DETAIL_REPORT_FILE_NAME $CONCMP_REPORT_FILE_NAME &>> /dev/null

if [ ! -z "$CURR_PID" ] ; then
	echo "" > $MODULE_DIR/$MODULE_NAME.pid
fi


exit 0