#!/usr/bin/env python
# -*- coding: utf-8 -*-

import ConfigParser
import sys
import os
import time
import base64
import hmac
import sha
import logging
import logging.config
import logging.handlers
import traceback
from enums import *
try:
    import DevClient
except ImportError:
    DevClient = None

class AUTH_TYPE():
    NO_AUTH = 0
    NORMAL_AUTH = 1
    SKYNET_AUTH = 2

class AgentConfig(object):
    CONFIGFILE         = '/usr/trend/commonlog/agent/config/log_agent.ini'
    COMMON_CONFIGFILE  = '/usr/trend/commonlog/config/CommonLog.ini'
    SYSEVENT_CONFIGFILE = '/usr/trend/commonlog/agent/config/sys_event_agent.ini'
    LOGGING_CONFIGFILE = '/usr/trend/commonlog/agent/config/logging.ini'
    DATA_DIR           = '/usr/trend/commonlog/data'
    UPLOAD_DATA_DIR    = 'agent/upload/'
    FAILED_DATA_DIR    = 'agent/failed/'


    EXCEPTION_CONFIG   = '/usr/trend/commonlog/agent/config/Except_list.ini'

    MIGRATION      = 0
    MIGRATION_IP   = '127.0.0.1'
    MIGRATION_DB_TYPE = 'IWSVA'

    PRODUCT        = Product.IWSS
    DEPLOY_MODE    = DeployMode.ALL_IN_ONE
    LOG_SERVER_ADDR= '127.0.0.1'
    LOG_SERVER_PORT= '6011'
    VERBOSE        = 0
    COMPRESS_STATE = 1
    PROTOCOL       = 'https'
    AUTH           = 0
    ACCOUNT        = ''
    PASSWORD       = ''

    LOG_LEVEL      = logging.INFO

    LATEST_UPLOAD_TIMESTAMP = 0

    CACHE_IWSS_ACCESS_LOG_LENGTH = 20000
    IWSS_VIOLATION_LOG_TIMEOUT = 60

    UPLOAD_STATUS  = True
    UPLOAD_MAX_LINES = 30000
    UPLOAD_MAX_TIMEOUT = 80
    UPLOAD_MAX_QUEUE = 60

    # log schemas
    UNIFIED_SCHEMA_ITEMS        = None
    IWSS_APP_SCHEMA_ITEMS       = None
    IWSS_ACCESS_SCHEMA_ITEMS    = None
    IWSS_VIOLATION_SCHEMA_ITEMS = None
    IWSS_CCCA_SCHEMA_ITEMS      = None
    ICS_ACCESS_SCHEMA_ITEMS     = None
    DEEP_ACCESS_SCHEMA_ITEMS    = None
    DEEP_VIOLATION_SCHEMA_ITEMS = None

    # exception list
    EXCEPTION_LIST              = None

    # other
    __inited = False
    @staticmethod
    def init():
        logging.debug('AgentConfig.init:')
        if AgentConfig.__inited:
            return

        '''Load agent configuration from config file.'''
        try:
            logging.config.fileConfig(AgentConfig.LOGGING_CONFIGFILE)
            config_reader = ConfigParser.ConfigParser()
            if os.path.exists(AgentConfig.SYSEVENT_CONFIGFILE):
                config_reader.read(AgentConfig.SYSEVENT_CONFIGFILE)
                AgentConfig.UPLOAD_STATUS= config_reader.getboolean('UPLOAD_STATUS', 'status')
            

            config_reader.read(AgentConfig.EXCEPTION_CONFIG)
            AgentConfig.EXCEPTION_LIST  = config_reader.get('EXCEPTLIST', 'json')

            config_reader.read(AgentConfig.CONFIGFILE)
            AgentConfig.PRODUCT         = config_reader.get('AGENT', 'product')
            AgentConfig.DEPLOY_MODE     = config_reader.getint('AGENT', 'mode')
            AgentConfig.LOG_SERVER_ADDR = config_reader.get('AGENT', 'log_server_addr')
            AgentConfig.LOG_SERVER_PORT = config_reader.get('AGENT', 'log_server_port')
            AgentConfig.LOG_REDUCTION   = config_reader.getint('AGENT', 'log_reduction')
            AgentConfig.VERBOSE         = config_reader.getint('AGENT', 'verbose')
            AgentConfig.DATA_DIR        = config_reader.get('AGENT', 'data_dir')
            AgentConfig.UPLOAD_MAX_LINES   = config_reader.getint('AGENT', 'upload_max_lines')
            AgentConfig.UPLOAD_MAX_TIMEOUT = config_reader.getint('AGENT', 'upload_max_timeout')
            if not os.path.exists(AgentConfig.DATA_DIR):
                os.mkdir(AgentConfig.DATA_DIR)
            try:
                AgentConfig.UPLOAD_MAX_QUEUE = config_reader.getint('AGENT', 'upload_max_queue')
            except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
                logging.info('fail to find the parameter upload_max_queue and set 60 by default: %s', e)
            

            
            AgentConfig.UPLOAD_DATA_DIR = config_reader.get('AGENT', 'upload_data_dir')
            AgentConfig.FAILED_DATA_DIR = config_reader.get('AGENT', 'failed_data_dir')
            AgentConfig.UPLOAD_DATA_DIR = os.path.join(AgentConfig.DATA_DIR, AgentConfig.UPLOAD_DATA_DIR)
            AgentConfig.FAILED_DATA_DIR = os.path.join(AgentConfig.DATA_DIR, AgentConfig.FAILED_DATA_DIR)
            if not os.path.exists(AgentConfig.UPLOAD_DATA_DIR):
                os.mkdir(AgentConfig.UPLOAD_DATA_DIR)
            if not os.path.exists(AgentConfig.FAILED_DATA_DIR):
                os.mkdir(AgentConfig.FAILED_DATA_DIR)

            AgentConfig.COMPRESS_STATE  = config_reader.getint('AGENT', 'compress_state')
            AgentConfig.CACHE_IWSS_ACCESS_LOG_LENGTH = config_reader.getint('AGENT', 'cache_iwss_access_log_length')
            AgentConfig.IWSS_VIOLATION_LOG_TIMEOUT   = config_reader.getint('AGENT', 'iwss_violation_log_timeout')
            AgentConfig.AUTH                         = config_reader.getint('AGENT', 'auth')
            if AgentConfig.AUTH:
                AgentConfig.ACCOUNT  = config_reader.get('AGENT', 'account')
                AgentConfig.PASSWORD = AgentConfig.decryptpassword(config_reader.get('AGENT', 'password'))

            AgentConfig.UNIFIED_SCHEMA_ITEMS        = config_reader.get('LOG_SCHEMA','unified_log').split(',')
            AgentConfig.IWSS_APP_SCHEMA_ITEMS       = config_reader.get('LOG_SCHEMA','iwss_app_log').split(',')
            AgentConfig.IWSS_ACCESS_SCHEMA_ITEMS    = config_reader.get('LOG_SCHEMA','iwss_access_log').split(',')
            AgentConfig.IWSS_VIOLATION_SCHEMA_ITEMS = config_reader.get('LOG_SCHEMA','iwss_violation_log').split(',')
            AgentConfig.IWSS_CCCA_SCHEMA_ITEMS      = config_reader.get('LOG_SCHEMA', 'iwss_ccca_log').split(',')
            AgentConfig.ICS_ACCESS_SCHEMA_ITEMS     = config_reader.get('LOG_SCHEMA', 'ics_access_log').split(',')
            AgentConfig.DEEP_ACCESS_SCHEMA_ITEMS    = config_reader.get('LOG_SCHEMA', 'deep_edge_access_log').split(',')
            AgentConfig.DEEP_VIOLATION_SCHEMA_ITEMS = config_reader.get('LOG_SCHEMA', 'deep_edge_violation_log').split(',')
           
            AgentConfig.MIGRATION_DB_TYPE = config_reader.get('AGENT','migration_db_type')
            

            ssl = config_reader.getint('AGENT', 'ssl')
            if ssl == 0:
                AgentConfig.PROTOCOL = 'http'

            if AgentConfig.VERBOSE == 1:
                AgentConfig.LOG_LEVEL = logging.DEBUG

            if AgentConfig.PRODUCT == "IWSS":
                IWSS_SYSTEM_CONFIG_FILE = '/etc/iscan/intscan.ini'
                config_reader.read(IWSS_SYSTEM_CONFIG_FILE)
                AgentConfig.REST_UI_PROTOL = config_reader.get('http', 'iscan_web_protocol')
                AgentConfig.REST_UI_PORT   = (config_reader.get('http', 'iscan_web_server')).split(':')[-1]

            logging.getLogger().setLevel(AgentConfig.LOG_LEVEL)
            logging.debug('Read config file successfully.')
            logging.debug('Product:                %s', AgentConfig.PRODUCT)
            logging.debug('DeployMode:             %s', AgentConfig.DEPLOY_MODE)
            logging.debug('LogServer:              %s', AgentConfig.LOG_SERVER_ADDR)
            logging.debug('LogPort:                %s', AgentConfig.LOG_SERVER_PORT)
            logging.debug('Verbose:                %s', AgentConfig.VERBOSE)
            logging.debug('CompressState:          %s', AgentConfig.COMPRESS_STATE)
            logging.debug('CacheIwssAccessLogLen:  %s', AgentConfig.CACHE_IWSS_ACCESS_LOG_LENGTH)
            logging.debug('IwssViolationLogTimeout:%s', AgentConfig.IWSS_VIOLATION_LOG_TIMEOUT)
            logging.debug('Protocol:               %s', AgentConfig.PROTOCOL)
            logging.debug('Auth:                   %s', AgentConfig.AUTH)
            logging.debug('Account:                %s', AgentConfig.ACCOUNT)
            logging.debug('Password:               %s', AgentConfig.PASSWORD)

            AgentConfig.__inited = True
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
            logging.info('Read agent config file error: %s', e)
            sys.exit(-1)

    @staticmethod
    def decryptpassword(encrypted_pass):
        cmd = "/usr/iwss/bin/encpw -d '%s'" % encrypted_pass
        password = os.popen(cmd).read().replace('\n', '')
        return password

    @staticmethod
    def getAuthHeader():
        headers = {}
        if AgentConfig.AUTH:
            date     = time.strftime("%a, %d %b %Y %X GMT", time.gmtime())
            password = base64.encodestring(hmac.new(AgentConfig.PASSWORD, date, sha).digest()).strip()
            authorization = '%s:%s' % (AgentConfig.ACCOUNT, password)
            headers = {'Date': date, 'Authorization': authorization}
        elif AgentConfig.AUTH == AUTH_TYPE.SKYNET_AUTH and DevClient:
            headers['VerifyMsg'] = DevClient.do_generate_verification_str()
        else:
            pass
        return headers

    @staticmethod
    def getUploadStatsConfig():
        # upload stats to Skynet: 1 -> yes, 0 -> no
        try:
            cfg = ConfigParser.ConfigParser()
            cfg.read(AgentConfig.COMMON_CONFIGFILE)
            return cfg.getint('stats_storage', 'upload_stats')
        except:
            logging.error(traceback.format_exc())
            return 0
