#!/bin/sh

if [ "$#" -eq 2 ] && [ "$1" = "--workdir" ]; then
    relative_path=$2
else
    relative_path=`dirname ${0}`
fi

absolute_path=`cd $relative_path >/dev/null; pwd ; cd - >/dev/null`
COMMON_LDAP_PATH=$absolute_path

krb5_config="${COMMON_LDAP_PATH}/krb5.conf"

read_ini_key()
{
    ini_file=$1
    ini_section=$2
    ini_key=$3

    ini_value=`sed -e 's/[[:space:]]*\=[[:space:]]*/=/g' -e '/^#/d' -e '/^$/d' \
        -e 's/[[:space:]]*$//' -e 's/^[[:space:]]*//' < $ini_file \
        | sed -n "/^\[$ini_section\]/,/^\s*\[/{/^[^;].*\=.*/p;}" | grep "$ini_key" | cut -d'=' -f2`
}

create_krb5_conf()
{
    ! [ -f "${COMMON_LDAP_PATH}/LdapSetting.ini" ] && return

    idx=0
    default_realm=""

    read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "LDAP_Setting" "DomainNum"
    domain_num=$ini_value

    while [ "$idx" -lt "$domain_num" ]; do

        read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "Domain_$idx" "Authentication"
        domain_auth[$idx]=$ini_value

        if [ "${domain_auth[$idx]}" = "kerberos" ]; then
            read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "Domain_$idx" "KDCServer"
            domain_kdc_server[$idx]=$ini_value

            read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "Domain_$idx" "KDCRealm"
            domain_kdc_realm[$idx]=$ini_value
            [ -z "$default_realm" ] && default_realm=$ini_value

            read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "Domain_$idx" "KDCDomain"
            domain_kdc_domain[$idx]=$ini_value
        fi

        idx=$((idx+1))
    done    


    {
        echo "#automatically generated krb5 configure"
        echo "" 
        echo "[libdefaults]"
        echo "  ticket_lifetime=900" 
        [ -n $default_realm ] && echo "  default_realm=$default_realm"
        echo "  clockskew=300"
        echo "  #dns_lookup_kdc=false"
        echo "  #dns_lookup_realm=false"
	echo "  default_keytab_name=FILE:${COMMON_LDAP_PATH}/iwsva.keytab"
        echo "" 
        echo "[realms]"
        echo ""
        
        idx=0
        while [ "$idx" -lt "$domain_num" ]; do

            if [ "${domain_auth[$idx]}" != "kerberos" ]; then
                idx=$((idx+1))
                continue
            fi

            field=1
            admin_server=""

            echo "  ${domain_kdc_realm[$idx]}={"
            while [ 1 ]; do
                server=`echo "${domain_kdc_server[$idx]}" | cut -d';' -f $field`
                if [ -n "$server" ]; then
                    echo "    kdc=$server"
                    [ -z "$admin_server" ] && admin_server=`echo $server | cut -d':' -f 1`
                fi

                [ -z "$server" -o "$server" = "${domain_kdc_server[$idx]}" ] && break
                field=$((field+1))
            done

            [ -n "$admin_server" ] && echo "    admin_server=$admin_server"
            echo "    default_domain=${domain_kdc_realm[$idx]}"
            echo "  }"
            echo ""

            idx=$((idx+1))
        done

        echo "[domain_realm]"
        
        idx=0
        while [ "$idx" -lt "$domain_num" ]; do

            if [ "${domain_auth[$idx]}" != "kerberos" ]; then
                idx=$((idx+1))
                continue
            fi

            echo "  .${domain_kdc_realm[$idx]}=${domain_kdc_domain[$idx]}"
            echo "  ${domain_kdc_realm[$idx]}=${domain_kdc_domain[$idx]}"
            echo ""
            idx=$((idx+1))
        done
    } > $krb5_config
}

create_krb5_conf
