#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import time
import string
import re
import os
import json
import logging
import urllib2
import codecs
import ConfigParser
import fcntl
from enums import *
from file_operate   import Configer
from agent_config   import AgentConfig
from base_converter import BaseConverter

#wesson add for client certificate enhance
def writeAppend(filename, data):
    logging.debug('ReadWriteFile.writeAppend:')
    try:
        if data == '':
            return False
        fp = open(filename,'a')
        #LockFile.lock_write(fp)
        fcntl.flock(fp, fcntl.LOCK_EX)
        fp.write(data)
        #LockFile.unlock(fp)
        fcntl.flock(fp, fcntl.LOCK_UN)
        fp.close()
        return True
    except:
        logging.exception('ReadWriteFile.writeAppend Error')
        return False
		

def getTok(str):
    index1 = str.find('\"')
    index2 = str.find('\"',index1+1,len(str))
    return str[index1+1:index2].lower()

def open_file(file_name, flag):
        if not os.path.exists(file_name):
                logger.error("file not exit: " + file_name)
                return None
        real_encoding = None
        encodings = ('utf-16', 'utf-8', 'gb2312')
        for encoding in encodings:
                fd = codecs.open(file_name, flag, encoding)
                try:
                    fd.readline()
                except UnicodeError:
                    fd.close()
                else:
                    real_encoding = encoding
                    fd.close()
                    break
        if not real_encoding:
            logger.error("what's the file encoding? i cannot figure out")
            return None
        #fd = codecs.open(file_name, flag, real_encoding)
        fd = open(file_name, flag)
        #logging.info(">>>>> encoding:%r",real_encoding)
        return fd, real_encoding

urlCategoryMappingFile = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'url_category.ini')
class IwssLogConverter(BaseConverter):

    CUSTOM_URL_CATEGORY = {}
    logformat = AgentConfig.IWSS_ACCESS_SCHEMA_ITEMS
    # mapping table when conveting the log
    # 1. iwss_app_log -> unified_log
    app2unified_dict              = {'CLF_UserName'  : 'username',
                                     'CLF_GroupName' : 'group_name',
                                     'CLF_AppID'     : 'protocol',
                                     'CLF_ClientIP'  : 'ip_addr',
                                     'CLF_Connection': 'connection',
                                     'CLF_OnlineTime': 'online_time',
                                     'CLF_Action'    : 'action',
                                     'CLF_PolicyName': 'policy_name',
                                     'CLF_InByte'    : 'in_throughput',
                                     'CLF_OutByte'   : 'out_throughput'}

    # 2. iwss_access_log -> unified_log
    access2unified_dict           = {'CLF_UserName'  : 'username',
                                     'CLF_URL'       : 'url',
                                     'CLF_Size'      : 'size',
                                     'CLF_Channel'   : 'protocol',
                                     'CLF_MimeType'  : 'mime_content',
                                     'CLF_ClientIP'  : 'client_ip',
                                     'CLF_ServerIP'  : 'server_ip',
                                     'CLF_Domain'    : 'domain',
                                     'CLF_FileName'  : 'file_name',
                                     'CLF_ProtocolAction'    : 'operation',
                                     'CLF_URLCategory'       : 'category',
                                     'CLF_CustomURLCategory' : 'category_type',
                                     'CLF_ModuleID': 'module_id',
                                     'CLF_ErrorCode': 'error_code',
                                     'CLF_ErrorParameters': 'error_para',
                                     'CLF_TimeStamp':'time_stamp'
                                     }
    if 'xforwardedfor' in logformat:
        access2unified_dict['CLF_XForWardedFor'] = 'xforwardedfor'
    if 'original_ip' in logformat:
        access2unified_dict['CLF_OriginalIP'] = 'original_ip'
    if 'referrer' in logformat:
        access2unified_dict['CLF_Referrer'] = 'referrer'
    if 'xrequestedwith' in logformat:
        access2unified_dict['CLF_XRequestedWith'] = 'xrequestedwith'
    if 'interruptedreason' in logformat:
        access2unified_dict['CLF_InterruptedReason'] = 'interruptedreason'

    # 3. iwss_violation_log -> unified_log
    violation2unified_dict        = {'CLF_UserName'    : 'username',
                                     'CLF_GroupName'   : 'group_name',
                                     'CLF_URL'         : 'url',
                                     'CLF_Channel'     : 'protocol',
                                     'CLF_FileName'    : 'file_name',
                                     'CLF_PolicyName'  : 'rule_name',
                                     'CLF_Size'              : 'size',
                                     'CLF_MimeType'          : 'mime_content',
                                     'CLF_ClientIP'          : 'client_ip',
                                     'CLF_ServerIP'          : 'server_ip',
                                     'CLF_Domain'            : 'domain',
                                     'CLF_ProtocolAction'    : 'operation'}

    # 4. iwss_ccca_log -> unified_log
    ccca2unified_dict             = {'CLF_UserName'   : 'username',
                                     'CLF_Channel'    : 'protocol',
                                     'CLF_URL'        : 'url',
                                     'CLF_Domain'     : 'domain',
                                     'CLF_PolicyName' : 'rule_name',
                                     'CLF_GroupName'  : 'group_name',
                                     'CLF_ClientIP'   : 'client_ip',
                                     'CLF_ServerIP'   : 'server_ip',
                                     'CLF_DestinationPort' : 'dest_port',
                                     'CLF_Action'     : 'filtering_action'}

    urlMappingReader = ConfigParser.ConfigParser()
    urlMappingReader.read(urlCategoryMappingFile)

    def _init_builtin_urlcategory(self):
        urlcategoryMappingFile = '/etc/iscan/urlfcMapping.ini'
        urlc = {}
        with open(urlcategoryMappingFile) as fp:
            while True:
                line = fp.readline()
                if not line:
                    break
                if line.startswith('#') or line.startswith('['):
                    continue
                urlcategoryId, urlcategoryName = line.split('=')
                urlc[urlcategoryId.strip()] = urlcategoryName.strip()
        return urlc

    def _init_builtin_appcategory(self):
        appcategoryMappingFile = '/etc/iscan/appcMapping.ini'
        appc = {}
        with open(appcategoryMappingFile) as fp:
            while True:
                line = fp.readline()
                if not line:
                    break
                if line.startswith('#') or line.startswith('['):
                    continue
                appcategoryId, appcategoryName, _ = line.split('=')
                appc[appcategoryId.strip()] = appcategoryName.strip()
        return appc

    def _init_builtin_action(self):
        actionMappingFile = '/usr/trend/commonlog/config/CLFDict.ini'
        cfg = ConfigParser.ConfigParser()
        cfg.read(actionMappingFile)
        actionc = {}
        for option in cfg.options('CLF_Action'):
            actionc[option] = cfg.get('CLF_Action', option)
        return actionc

    def _init_builtin_risklevel(self):
        actionMappingFile = '/usr/trend/commonlog/config/CLFDict.ini'
        cfg = ConfigParser.ConfigParser()
        cfg.read(actionMappingFile)
        risklevelc = {}
        for option in cfg.options('CLF_RiskLevel'):
            risklevelc[option] = cfg.get('CLF_RiskLevel', option)
        return risklevelc


    def _init_rulename(self):
        src_fd,coding = open_file('/usr/iwss/L10N_tool/en_urlblocking.txt','r')
        line = src_fd.readline()
        self.tok_DestinationPort = ''
        self.tok_ClientQuota = ''
        self.tok_ClientIP = ''
        while line:
            if line.find('tok_DestinationPort') != -1:
                self.tok_DestinationPort = getTok(line)
            elif line.find('tok_ClientIP') != -1:
                self.tok_ClientIP = getTok(line)
            elif line.find('tok_ClientQuota') != -1:
                self.tok_ClientQuota = getTok(line)
            line = src_fd.readline()
        src_fd.close()

    def __init__(self):
        '''initialize the log schema'''
        self.CLF_ProductID = self._get_product_id()
        self.CLF_ServerName = self._get_servername()
        self.CLF_TimeZone = self._get_timezone()

        self._unified_schema_items        = AgentConfig.UNIFIED_SCHEMA_ITEMS
        self._iwss_app_schema_items       = AgentConfig.IWSS_APP_SCHEMA_ITEMS
        self._iwss_access_schema_items    = AgentConfig.IWSS_ACCESS_SCHEMA_ITEMS
        self._iwss_violation_schema_items = AgentConfig.IWSS_VIOLATION_SCHEMA_ITEMS
        self._iwss_ccca_schema_items      = AgentConfig.IWSS_CCCA_SCHEMA_ITEMS
        if AgentConfig.MIGRATION_DB_TYPE == 'ARM':
            self._iwss_app_schema_items.append('host_ip')
            self._iwss_access_schema_items.append('host_ip')
            self._iwss_violation_schema_items.append('host_ip')
            self._iwss_ccca_schema_items.append('host_ip')

        IwssLogConverter.CUSTOM_URL_CATEGORY = self._get_iwss_custom_url_category()
        self.updateCustomCategoryTime        = time.time()

        self.l10nUrlCate = self._init_builtin_urlcategory()
        self.l10nAppCate = self._init_builtin_appcategory()
        self.l10nAction = self._init_builtin_action()
        self.l10nRiskLevel = self._init_builtin_risklevel()
        self._init_rulename()

    def doL10n(self, log):
        logItems = log.split('\t')
        l10nColumnName = ['CLF_URLCategory', 'CLF_AppID', 'CLF_Action', 'CLF_RiskLevel']
        l10nResources = [self.l10nUrlCate, self.l10nAppCate, self.l10nAction, self.l10nRiskLevel]
        for i in xrange(len(l10nColumnName)):
            columnName = l10nColumnName[i]
            resource = l10nResources[i]
            # list.index(value): if value is not existed, valueError will be raised.
            # but columnName must exist in this case.
            idx = self._unified_schema_items.index(columnName)
            value = logItems[idx]
            if columnName != 'CLF_URLCategory':
                if value:
                    if value in resource:
                        logItems[idx] = resource[value]
                    else:
                        # missing l10n resource
                        logItems[idx] = 'unknown'
            else:
                idx2 = self._unified_schema_items.index('CLF_CustomURLCategory')
                value2 = logItems[idx2]
                # if value(CLF_URLCategory) is not '', then value2(CLF_CustomURLCategory) is must ''
                # if value(CLF_URLCategory) is '', then value2(CLF_CustomURLCategory) is not ''
                if value:
                    if value in resource:
                        # do10n on 'CLF_URLCategory' column
                        logItems[idx] = resource[value]
                    else:
                        logItems[idx] = 'unknown'
                elif value2:
                    # Move CLF_CustomURLCategory to CLF_URLCategory, CLF_CustomURLCategory is no use.
                    logItems[idx] = value2
                    logItems[idx2] = ''
        return '\t'.join(logItems)

    def _get_product_id(self):
        # Command: "/usr/iwss/iwssd -v"
        # IWSVA 5.1-SP1_Build_Linux_1231 $Date: 08/22/2012 18:23:34 PM$ -> IWSVA51SP1
        # IWSVA 5.5_Build_Linux_1287 $Date: 07/29/2011 19:30:08 PM$     -> IWSVA56
        product_id = "IWSVA"
        try:
            version_information = os.popen('/usr/iwss/iwssd -v').readlines()
            product_id = (re.search(r'(.*)(_Build.*)', str(version_information[0]))).group(1)
            product_id = product_id.replace(' ', '').replace('.', '').replace('-','')
        except Exception as e:
            pass
        return product_id

    def _get_iwss_app_msytype(self, app_action):
        '''app log: 0 - allow; 1 - block '''
        app_msgtype = MsgType.UNKNOWN
        app_action = string.strip(app_action)
        if app_action == '0':
            app_msgtype = MsgType.APP_ACCESS_LOG
            #app_msgtype = MsgType.APP_BLOCK_LOG
        else:
            app_msgtype = MsgType.APP_BLOCK_LOG
            #app_msgtype = MsgType.APP_ACCESS_LOG
        return app_msgtype

    def _handle_special_column(self, unified_log):
        unified_log['CLF_MimeType'] = unified_log['CLF_MimeType'].replace('*', '')
        unified_log['CLF_URLCategory'] = unified_log['CLF_URLCategory'].replace('-1', '0')


    def _get_iwss_violation_msgtype(self, unified_log, log_dict):
        '''get violation msgtype for iwss'''
        violation_type = MsgType.UNKNOWN

        scan_type        = string.strip(string.lower(log_dict['scan_type']))
        malicious_entity = string.strip(string.lower(log_dict['malicious_entity']))
        rule_name        = string.strip(string.lower(log_dict['rule_name']))
        blocked_by       = string.strip(string.lower(log_dict['blocked_by']))

        # 1. virus (spyware/grayware, bot, virus infected temporary, virus) (action: block, quarantine)
        if scan_type == 'virus scan':
            if malicious_entity == 'spyware' or malicious_entity == 'grayware':
                violation_type = MsgType.SPYWARE_LOG
            elif blocked_by ==  'rule':
                violation_type = MsgType.VIRUS_INFECTED_TEMPORARY_BLOCK_LOG
            else:
                violation_type = MsgType.VIRUS_LOG
        # bot (action: block, monitor)
        elif scan_type == 'apt detection':
            violation_type = MsgType.APT_DETECTION
        elif scan_type == 'bot detection':
            violation_type = MsgType.BOT_DECTION_LOG
        # 2. file type block (action: block)
        elif scan_type == 'normal':
            violation_type = MsgType.FILE_TYPE_BLOCK_LOG
        # 3. http inspection (action: block, monitor)
        elif scan_type == 'http inspection':
            violation_type = MsgType.HTTP_INSPECTION_LOG
        # 4. dlp filter (action: block, monitor)
        elif scan_type == 'dlp filter':
            violation_type = MsgType.DLP_LOG
        # 5. java scan (action: block)
        elif scan_type == 'javascan':
            violation_type = MsgType.JAVASCAN_LOG
        # 6. URL filtering (action: block, monitor, warn, warncontinue, timequota, override)
        elif scan_type == 'content filter':
            violation_type = MsgType.URL_FILTERING_LOG
        # 7. WRS (action: block)
        elif scan_type == 'web reputation' or scan_type == 'anti-phishing' or scan_type == 'anti-pharming' or scan_type == 'web reputation - (pass)':
            violation_type = MsgType.WRS_LOG
        # 10. global url blocking (via phish pattern) (action: block)
        elif scan_type == 'phishtrap':
            violation_type = MsgType.GLOBAL_URL_BLOCKING_PHISH_PATTERN_LOG
        elif scan_type == 'user defined':
            # 8. access quota (action: block)
            rule_name = rule_name.lower()
            #if re.search(tok_ClientQuota, rule_name) is not None:
            #    violation_type = MsgType.ACCESS_QUOTA_LOG
            # 9. access control (client ip acl, destination port restriction) (action: block)
            if rule_name == self.tok_ClientQuota:
                violation_type = MsgType.ACCESS_QUOTA_LOG
            elif rule_name == self.tok_ClientIP:
                violation_type = MsgType.ACCESS_CONTROL_CLIENT_IP_LOG
            elif rule_name == self.tok_DestinationPort:
                violation_type = MsgType.ACCESS_CONTROL_DESTINATION_PORTS_LOG
            # 10. global url blocking (via local list)
            else:
                violation_type = MsgType.GLOBAL_URL_BLOCKING_LOCAL_LIST_LOG

        # Rule name of DLP log is stored in field ('entity_name')
        if violation_type == MsgType.DLP_LOG:
            unified_log['CLF_RuleName']       = log_dict['entity_name']
            unified_log['CLF_MatchedContent'] = log_dict['malicious_entity']
        elif violation_type == MsgType.URL_FILTERING_LOG:
            category = log_dict['category']
            if IwssLogConverter.urlMappingReader.has_option('category', category):
                unified_log['CLF_URLCategory'] = IwssLogConverter.urlMappingReader.get('category', category)
                unified_log['CLF_CustomURLCategory'] = ''
            else:
                unified_log['CLF_URLCategory'] = ''
                unified_log['CLF_CustomURLCategory'] = category
        # Rule name of other logs are soted in field ('category')
        else:
            unified_log['CLF_RuleName']    = log_dict['category']
            unified_log['CLF_MalwareName'] = log_dict['entity_name']
        return violation_type

    def _get_iwss_action(self, iwss_action, iwss_filtering_action, scan_type = None):
        iwss_action = string.lower(iwss_action)

        action = Action.BLOCK
        if re.search('quarantine', iwss_action) is not None:
            action = Action.QUARANTINE
        else:
            if iwss_filtering_action == '0' and scan_type == 'web reputation - (pass)':
                action = Action.MONITOR
            elif iwss_filtering_action == '0':
                action = Action.BLOCK
            else:
                action = iwss_filtering_action
        return action

    def convert_app(self, logline, RBACdict=None):
        if AgentConfig.MIGRATION == 1:
            self.CLF_ServerName = self._get_servername()
            self.CLF_ProductID = ""

        logitems = logline.replace('\n', '').split('\t')
        log_dict = dict(zip(self._iwss_app_schema_items, logitems))
        # init a unified log dict
        unified_log = collections.defaultdict(str).fromkeys(self._unified_schema_items, '')
        unified_log['CLF_ProductID']  = self.CLF_ProductID
        unified_log['CLF_DeviceName'] = self.CLF_ServerName
        unified_log['CLF_TimeZone']   = self.CLF_TimeZone
        unified_log['CLF_CompanyID']        = '0'
        unified_log['CLF_MsgType']          = str(self._get_iwss_app_msytype(log_dict['action']))
        unified_log['CLF_UTCTimeStamp']     = str(self._timestring2UTCtimestamp(log_dict['log_time']))
        unified_log['CLF_SessionStartTime'] = str(self._timestring2UTCtimestamp(log_dict['session_start_time']))

        log2unified_dict = IwssLogConverter.app2unified_dict
        for key in log2unified_dict:
            unified_log[key] = log_dict[log2unified_dict[key]]

        if RBACdict:
            if log_dict['username'].decode('UTF-8') in RBACdict:
                unified_log['CLF_RBACBitmap'] = str(RBACdict[log_dict['username'].decode('UTF-8')])
        if AgentConfig.MIGRATION == 1:
            if AgentConfig.MIGRATION_DB_TYPE == 'ARM':
                unified_log['CLF_DeviceName'] = log_dict['host_ip']

        self._handle_special_column(unified_log)

        unified_log_quene = [unified_log[self._unified_schema_items[i]] for i in xrange(len(self._unified_schema_items))]
        return unified_log['CLF_CompanyID'], unified_log['CLF_UTCTimeStamp'], '\t'.join(unified_log_quene) + '\n'

    def convert_access(self, logline, RBACdict=None):
        if AgentConfig.MIGRATION == 1:
            self.CLF_ServerName = self._get_servername()
            self.CLF_ProductID = ""

        if abs(self.updateCustomCategoryTime - time.time()) >= 300:
            IwssLogConverter.CUSTOM_URL_CATEGORY = self._get_iwss_custom_url_category()
            self.updateCustomCategoryTime = time.time()

        logitems = logline.replace('\n', '').split('\t')
        log_dict = dict(zip(self._iwss_access_schema_items, logitems))
        # init a unified log dict
        unified_log = collections.defaultdict(str).fromkeys(self._unified_schema_items, '')
        unified_log['CLF_ProductID']  = self.CLF_ProductID
        unified_log['CLF_DeviceName'] = self.CLF_ServerName
        unified_log['CLF_TimeZone']   = self.CLF_TimeZone
        unified_log['CLF_CompanyID']        = '0'
        unified_log['CLF_MsgType']          = str(MsgType.ACCESS_LOG)
        unified_log['CLF_UTCTimeStamp']     = str(self._timestring2UTCtimestamp(log_dict['date_field']))

        log2unified_dict = IwssLogConverter.access2unified_dict
        for key in log2unified_dict:
            if log_dict.has_key(log2unified_dict[key]):
                unified_log[key] = log_dict[log2unified_dict[key]]
            else:
                unified_log[key] = ''
            #unified_log[key] = log_dict[log2unified_dict[key]]

        if RBACdict:
            if log_dict['username'].decode('UTF-8') in RBACdict:
                unified_log['CLF_RBACBitmap'] = str(RBACdict[log_dict['username'].decode('UTF-8')])
        if log_dict['category_type'] != '0':
            if not log_dict['category'] in IwssLogConverter.CUSTOM_URL_CATEGORY:
                IwssLogConverter.CUSTOM_URL_CATEGORY = self._get_iwss_custom_url_category()

            if IwssLogConverter.CUSTOM_URL_CATEGORY.has_key(log_dict['category']):
                unified_log['CLF_URLCategory'] = ''
                unified_log['CLF_CustomURLCategory'] = IwssLogConverter.CUSTOM_URL_CATEGORY[log_dict['category']]
            else:
                unified_log['CLF_URLCategory'] = ''
                unified_log['CLF_CustomURLCategory'] = "Custom Category"
        else:
            unified_log['CLF_URLCategory'] = log_dict['category']
            unified_log['CLF_CustomURLCategory'] = ''

        if AgentConfig.MIGRATION == 1:
             if AgentConfig.MIGRATION_DB_TYPE == 'ARM':
                 unified_log['CLF_DeviceName'] = log_dict['host_ip']

        self._handle_special_column(unified_log)

        unified_log_quene = [unified_log[self._unified_schema_items[i]] for i in xrange(len(self._unified_schema_items))]
		
        if log_dict.has_key('error_code'):
            if log_dict['error_code'] == '4008':
                if log_dict.has_key('username') == False:
                    logging.debug('username null, use client ip')
                else:
                    log_dict['username'] = log_dict['client_ip']

                filename = os.path.join("/usr/iwss/sql/postgresql/", "client_certifcate_error")
                faild_data = log_dict['username'] + '|' + log_dict['client_ip'] + '|' + log_dict['server_ip'] + '|' + log_dict['domain'] + '|' + log_dict['date_field'] +'\n'
                logging.debug('filepath:%s, data:%s', filename, faild_data)
                writeAppend(filename,faild_data)
                    #with open(filename, 'a') as fp:
                    #fp.write(faild_data)
                    #fp.close()
                    #logging.debug('wesson protocal %s',  log_dict['protocol'])
                    #logging.debug('wesson client_ip %s', log_dict['client_ip'])    
                    #logging.debug('wesson domain %s',    log_dict['domain'])
                    #logging.debug('wesson server_ip %s', log_dict['server_ip'])
                    #logging.debug('wesson timestamp:%s', log_dict['date_field'])
                   #logging.debug('wesson filename %s',    filename)
           
        return unified_log['CLF_CompanyID'], unified_log['CLF_UTCTimeStamp'], '\t'.join(unified_log_quene) + '\n'

    def convert_violation(self, logline, RBACdict=None):
        if AgentConfig.MIGRATION == 1:
            self.CLF_ServerName = self._get_servername()
            self.CLF_ProductID = ""

        logitems = logline.replace('\n', '').split('\t')
        log_dict = dict(zip(self._iwss_violation_schema_items, logitems))
        # init a unified log dict
        unified_log = collections.defaultdict(str).fromkeys(self._unified_schema_items, '')
        unified_log['CLF_ProductID']    = self.CLF_ProductID
        unified_log['CLF_DeviceName']   = self.CLF_ServerName
        unified_log['CLF_TimeZone']     = self.CLF_TimeZone
        unified_log['CLF_CompanyID']    = '0'
        unified_log['CLF_MsgType']      = str(self._get_iwss_violation_msgtype(unified_log, log_dict))
        #scan_type        = string.strip(string.lower(log_dict['scan_type']))
        unified_log['CLF_Action']       = str(self._get_iwss_action(log_dict['action'], log_dict['filtering_action'], string.strip(string.lower(log_dict['scan_type']))))
        unified_log['CLF_UTCTimeStamp'] = str(self._timestring2UTCtimestamp(log_dict['date_field']))

        log2unified_dict = IwssLogConverter.violation2unified_dict
        for key in log2unified_dict:
            unified_log[key] = log_dict[log2unified_dict[key]]

        if RBACdict:
            if log_dict['username'].decode('UTF-8') in RBACdict:
                unified_log['CLF_RBACBitmap'] = str(RBACdict[log_dict['username'].decode('UTF-8')])

        if log_dict['urlcategory_type'] != '0':
            unified_log['CLF_URLCategory'] = ''
            unified_log['CLF_CustomURLCategory'] = log_dict['category']
        else:
            unified_log['CLF_URLCategory'] = log_dict['urlcategory']
            unified_log['CLF_CustomURLCategory'] = ''

        if AgentConfig.MIGRATION == 1:
            if AgentConfig.MIGRATION_DB_TYPE == 'ARM':
                unified_log['CLF_DeviceName'] = log_dict['host_ip']

        self._handle_special_column(unified_log)

        unified_log_quene = [unified_log[self._unified_schema_items[i]] for i in xrange(len(self._unified_schema_items))]
        return unified_log['CLF_CompanyID'], unified_log['CLF_UTCTimeStamp'], '\t'.join(unified_log_quene) + '\n'

    CCCA_RULENAME_DICT = { '0': 'global url',
                           '1': 'global ip',
                           '2': 'custom url',
                           '3': 'custom ip' }
    def _get_iwss_ccca_rulename(self, ccca_source):
        return self.CCCA_RULENAME_DICT[ccca_source]

    RISK_LEVEL_DICT    = {'0': 'unknown',
                          '1': 'low',
                          '2': 'medium',
                          '3': 'high' }
    def _get_iwss_ccca_risklevel(self, risk_level):
        if risk_level not in self.RISK_LEVEL_DICT:
            return '0'
        else:
            return risk_level

    def convert_ccca(self, logline, RBACdict=None):
        if AgentConfig.MIGRATION == 1:
            self.CLF_ServerName = self._get_servername()
            self.CLF_ProductID = ""

        logitems = logline.replace('\n', '').split('\t')
        log_dict = dict(zip(self._iwss_ccca_schema_items, logitems))
        # init a unified log dict
        unified_log = collections.defaultdict(str).fromkeys(self._unified_schema_items, '')
        unified_log['CLF_ProductID']    = self.CLF_ProductID
        unified_log['CLF_DeviceName']   = self.CLF_ServerName
        unified_log['CLF_TimeZone']     = self.CLF_TimeZone
        unified_log['CLF_CompanyID']    = '0'
        unified_log['CLF_MsgType']      = str(MsgType.CCCA_LOG)
        unified_log['CLF_UTCTimeStamp'] = str(self._timestring2UTCtimestamp(log_dict['date_field']))
        unified_log['CLF_RuleName']     = self._get_iwss_ccca_rulename(log_dict['ccca_source'])
        unified_log['CLF_RiskLevel']    = self._get_iwss_ccca_risklevel(log_dict['risk_level'])

        log2unified_dict = IwssLogConverter.ccca2unified_dict
        for key in log2unified_dict:
            unified_log[key] = log_dict[log2unified_dict[key]]

        if RBACdict:
            if log_dict['username'].decode('UTF-8') in RBACdict:
                unified_log['CLF_RBACBitmap'] = str(RBACdict[log_dict['username'].decode('UTF-8')])
        if AgentConfig.MIGRATION == 1:
            if AgentConfig.MIGRATION_DB_TYPE == 'ARM':
                unified_log['CLF_DeviceName'] = log_dict['host_ip']

        unified_log_quene = [unified_log[self._unified_schema_items[i]] for i in xrange(len(self._unified_schema_items))]
        return unified_log['CLF_CompanyID'], unified_log['CLF_UTCTimeStamp'], '\t'.join(unified_log_quene) + '\n'

    def _get_iwss_custom_url_category(self):
        URL  = '%s://127.0.0.1:%s/rest/commonlog/log_agent/custom_category' % (AgentConfig.REST_UI_PROTOL,
                                                                               AgentConfig.REST_UI_PORT)
        result = IwssLogConverter.CUSTOM_URL_CATEGORY
        try:
            resp = urllib2.urlopen(URL)
            temp = json.loads(resp.read())
            for id in temp:
                result[id] = temp[id].encode('utf-8')
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.exception('IwssReader get custom url category error: %s', e)
        return result
