#!/bin/sh
#
# chkconfig: 2345 08 92
# description: Starts, stops and saves iwsva firewall
#

export PATH=/sbin:/usr/sbin:$PATH

IWSS_BIN=/usr/iwss
IWSS_DATA=/var/iwss

MAIL_CONF=/opt/trend/imss/config/imss.ini
. $IWSS_BIN/bin/ini_util.sh
. $IWSS_BIN/iwsvanetkmod.sh

INTSCAN_CONF=$IWSS_DATA/intscan.ini
HTTP_PNI=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni
ICAP_PNI=$IWSS_DATA/IWSSPIProtocolIcap.pni
FTP_PNI=$IWSS_DATA/IWSSPIProtocolFtp.pni
NETWORK_CONF=$IWSS_DATA/network.ini
ACL_CONF=$IWSS_DATA/AdminACL.ini
CACHE_CONF=$IWSS_DATA/iwssCache.ini
TMUFE_CONF=$IWSS_DATA/urlfcIFX.ini
CUSTOM_CONF=$IWSS_DATA/iwsvafw_custom.ini
REVERSEPROXY_CONF=$IWSS_DATA/reverse_proxy_settings.ini

# http or icap
HTTP_MODE=http

# default configuration
# 0 indicates port is disabled
HTTP_PORT=""
FTP_PORT=""

GUEST_PORT="0"      # guest port is 8081

SSL_PORT=""

Reverse_Proxy_SSL_PORT=""
SECONDARY_PORT="0"

INTERESTED_HTTP_PORTS=""

SSH_PORT="0"
SERVER_FARM_PORT="1444"
POSTGRESQL_PORT="0" # postgresql port is 5432

TOMCAT_PORT="1812"
SCAN_BEFORE_DELIVER_PORT="9091"

ENABLE_PING="no"

SOCKS5_PROXY_PORT="0"
SNMP_PORT="161"
NTP_PORT="123"

ATS_PORT="0"

ENABLE_CACHE="no"
CACHE_SECTION="Cache"

SELF_PROXY="yes"
Self_Proxy_Mode="yes"

UPSTREAM_PROXY_PORT=""
WEB_SERVER_PORT="80"

RATING_SERVER=""
AU_SERVER=""

SMTP_PORT="25"
POP3_PORT="110"

BRIDGEMODE=$((`grep "^bridgemode=" $INTSCAN_CONF | sed -e 's/^bridgemode=//'`))

FW_ORG=/etc/init.d/iptables
FW_SYS=/etc/init.d/iptables_sys
FW_ORG_V6=/etc/init.d/ip6tables
FW_SYS_V6=/etc/init.d/ip6tables_sys

IWSVAFW=$IWSS_BIN/iwsvafw.sh

IPTABLE()
{
	if [ $# -lt 1 ]; then
		return 1
	fi

	case $1 in
	ipv4)
		shift
		iptables $@
		return 0
		;;
	ipv6)
		shift
		ip6tables $@
		return 0
		;;
	esac

	iptables $@
	ip6tables $@
}

read_config() {
	# MNG
#	MNG="`/usr/iwss/iwsvanetwork.sh get ip interface`"

	# Out-of-band management port
	SEPARATED_MGMT="no"
	F_ini_get $NETWORK_CONF 'network' '^separate_mgmt_port' > /dev/null
	if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
		SEPARATED_MGMT=$_R
	fi

#	echo "read_config_mgmt_enable="$SEPARATED_MGMT

	# Data port interface
	DATA_IF=
	DATA_IF_EX=
	if [ $BRIDGEMODE -eq 1 ] ; then
		F_ini_get $NETWORK_CONF 'network' '^vid' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] && [ "$_R" != "0" ] ; then
			DATA_IF="br0.$_R"
		else
			DATA_IF="br0"
		fi
	else
		F_ini_get $NETWORK_CONF 'network' '^ifname_proxy' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			DATA_IF="$_R"
		fi

		F_ini_get $NETWORK_CONF 'network' '^ifname_proxy_internal' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			DATA_IF="$DATA_IF $_R"
		fi

		# don't allow listening on external nic
		F_ini_get $NETWORK_CONF 'network' '^ifname_proxy_external' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			DATA_IF_EX="$_R"
		fi
	fi


	# get the separate management interface ip if it existed
	SEPARATED_MGMT_ADDR=""

	# Management port interface
	if [ "$SEPARATED_MGMT" = "no" ] ; then
		MGMT_IF=$DATA_IF
	else
		F_ini_get $NETWORK_CONF 'network' '^ifname_mgmt' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			MGMT_IF=$_R

			SEPARATED_MGMT_ADDR=`/sbin/ifconfig $MGMT_IF | grep -E "inet addr:([0-9]{1,3}\.){3}[0-9]{1,3}" -o | cut -d':' -f 2`
		fi
	fi

	# http/icap port and interested ports
	F_ini_get $INTSCAN_CONF 'main' 'protocol_config_path' > /dev/null
	if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
		ini_file=$_R

		if [ "`echo $ini_file | grep Icap`" != "" ]; then
			HTTP_MODE=icap
		else
			HTTP_MODE=http
		fi

		F_ini_get $ini_file 'main' '^port' > /dev/null
		if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
			HTTP_PORT=$_R
		fi

		if [ "$HTTP_MODE" = "http" ]; then
			F_ini_get $ini_file 'main' '^tproxy_port' > /dev/null
			if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
				HTTP_PORT="$HTTP_PORT $_R"
			fi
		fi
	fi

	if [ "$HTTP_MODE" = "http" ]; then
		PORT_LIST=`grep wccp_additional_http_port $HTTP_PNI | grep -v "#" | awk -F= '{print $2}' | awk -F"," '{print $1 " " $2 " " $3 " " $4 " " $5}'`
		HTTP_PORT="$HTTP_PORT $PORT_LIST"
	fi

	if [ "$HTTP_MODE" = "http" ]; then
		F_ini_get $HTTP_PNI 'main' '^interested_port' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			INTERESTED_HTTP_PORTS=$_R
		fi
	fi

	if [ "$HTTP_MODE" = "http" ]; then
		# secondary port
		if [ $BRIDGEMODE -ne 1 ] ; then
			F_ini_get $INTSCAN_CONF 'main' 'protocol_config_path' > /dev/null
			if [ "$?" = "0" ] && [ "$_R" != "" ] ; then

				F_ini_get $_R 'plug-in' '^name' > /dev/null
				if [ "$?" = "0" ] && [ "$_R" = "\"IWSS HTTP proxy\"" ] ; then

					F_ini_get $HTTP_PNI 'http' '^self_proxy' > /dev/null
					if [ "$?" = "0" ] && [ "$_R" =  "reverse" ] ; then

						F_ini_get $HTTP_PNI 'main' '^secondaryport' > /dev/null
						if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
							SECONDARY_PORT=$_R
						fi
					fi
				fi
			fi
		fi
	fi

	# ftp port
	F_ini_get $FTP_PNI 'main' '^port' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		FTP_PORT=$_R
	fi

	F_ini_get $FTP_PNI 'main' '^tproxy_port' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		FTP_PORT="$FTP_PORT $_R"
	fi

	PORT_LIST=`grep pasv_listen_port_list $FTP_PNI | grep -v "#" | awk -F= '{print $2}' | awk -F"," '{print $1 " " $2 " " $3 " " $4 " " $5}'`
	FTP_PORT="$FTP_PORT $PORT_LIST"

	PORT_LIST=`grep wccp_additional_ftp_port $HTTP_PNI | grep -v "#" | awk -F= '{print $2}' | awk -F"," '{print $1 " " $2 " " $3 " " $4 " " $5}'`
	FTP_PORT="$FTP_PORT $PORT_LIST"

	# ssl port
	if [ "$HTTP_MODE" = "http" ]; then
		F_ini_get $HTTP_PNI 'main' '^sslport' > /dev/null
		if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
			SSL_PORT=$_R
		fi

		F_ini_get $HTTP_PNI 'main' '^tproxy_sslport' > /dev/null
		if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
			SSL_PORT="$SSL_PORT $_R"
		fi

		PORT_LIST=`grep wccp_additional_https_port $HTTP_PNI | grep -v "#" | awk -F= '{print $2}' | awk -F"," '{print $1 " " $2 " " $3 " " $4 " " $5}'`
		SSL_PORT="$SSL_PORT $PORT_LIST"
	else
		F_ini_get $ICAP_PNI 'main' '^icaps_enable' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" = "yes" ] ; then
			F_ini_get $ICAP_PNI 'main' '^icaps_port' > /dev/null
			if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
				SSL_PORT=$_R
			fi
		fi
	fi

	# Reverse Proxy SSL Port
	if [ "$HTTP_MODE" = "http" ]; then
		F_ini_get $HTTP_PNI 'main' '^secondaryport' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			Reverse_Proxy_SSL_PORT=$_R
		fi

		# Proxy Mode
		F_ini_get $HTTP_PNI 'http' '^self_proxy' > /dev/null
		if [ "$?" = "0" ] && [ "$_R" != "" ] ; then
			Self_Proxy_Mode=$_R
		fi

		if [ $Self_Proxy_Mode == "reverse" ]; then
			F_ini_get $REVERSEPROXY_CONF 'https' '^port' > /dev/null
			Reverse_Proxy_SSL_PORT="$Reverse_Proxy_SSL_PORT $_R"
		fi
	fi

	#if mail daemon is disabled, do not get smtp and pop3 port number
	_isMailEnabled=`cat /opt/trend/imss/config/imss.ini | grep mail_enabled | cut -d'=' -f2 | tr -d ' '`
	if [ "$_isMailEnabled" = "no" ] ; then
		SMTP_PORT="0"
		POP3_PORT="0"
	else
		# smtp port
		F_ini_get $MAIL_CONF 'socket_1' 'proxy_port' > /dev/null
		if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
			SMTP_PORT=$_R
		fi


		# pop3  port
		F_ini_get $MAIL_CONF 'socket_2' 'proxy_port' > /dev/null
		if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
			POP3_PORT=$_R
		fi
	fi

	# ssh port
	SSH_STATUS=`chkconfig --list sshd | cut -f5 | cut -d: -f2`
	if [ "$SSH_STATUS" = "on" ] ; then
		SSH_PORT=`grep "^Port" /etc/ssh/sshd_config | cut -d' ' -f2`
		if [ "$SSH_PORT" = "" ] ; then
			SSH_PORT=22	# Not specified. Use default port 22
		fi
	fi

	# guest port
	if [ "$HTTP_MODE" = "http" ]; then
		if [ $BRIDGEMODE -ne 1 ] ; then
			F_ini_get $HTTP_PNI 'http' '^self_proxy' > /dev/null
			if [ "$?" = "0" ] && [ "$_R" != "reverse" ] ; then
				F_ini_get $INTSCAN_CONF 'http' '^guest_user_login' > /dev/null
				if [ "$?" = "0" ] && [ $_R = yes ] ; then
					F_ini_get $HTTP_PNI 'main' '^guestport' > /dev/null
					if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
						GUEST_PORT=$_R
					fi
				fi
			fi
		fi
	fi

	# server farm port
	F_ini_get $INTSCAN_CONF 'Server-farm' '^port' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		SERVER_FARM_PORT=$_R
	fi

	# postgresql port
	F_ini_get $NETWORK_CONF 'network' '^postgresql_port' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		POSTGRESQL_PORT=$_R
	fi

	# tomcat port
	F_ini_get $INTSCAN_CONF 'http' '^iscan_web_server' > /dev/null
	if [ "$?" = "0" ] ; then
		_R=`echo $_R | awk -F":" '{print $NF}'`
		[ $_R -gt 0 ] && TOMCAT_PORT=$_R
	fi

		F_ini_get $INTSCAN_CONF 'http' '^scan_before_deliver_server' > /dev/null
		if [ "$?" = "0" ] ; then
			_R=`echo $_R | cut -d':' -f2`
			[ $_R -gt 0 ] && SCAN_BEFORE_DELIVER_PORT=$_R
	fi

	# socks5 proxy port
	SOCKS5_PROXY_ENABLED=`/usr/bin/iniconfig -r -f $NETWORK_CONF -s network -o socks5_proxy_enabled`
	if [ "xx$SOCKS5_PROXY_ENABLED" = "xxyes" ]; then
		SOCKS5_PROXY_PORT=`/usr/bin/iniconfig -r -f $NETWORK_CONF -s network -o socks5_proxy_port`
	fi

	# ping
	F_ini_get $NETWORK_CONF 'network' '^enable_ping' > /dev/null
	if [ "$?" = "0" ] && [ "$_R" = "yes" ] ; then
		ENABLE_PING=$_R
	fi

	F_ini_get $NETWORK_CONF 'network' '^enable_ping_mgmt' > /dev/null
	if [ "$?" = "0" ] && [ "$_R" = "yes" ] ; then
		ENABLE_PING_MGMT=$_R
	fi

	# snmp
	F_ini_get $INTSCAN_CONF 'snmp' '^svc_port' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		SNMP_PORT=$_R
	fi

	SELF_PROXY=$(/usr/bin/iniconfig -r -f $HTTP_PNI -s "http" -o "self_proxy")
	if [[ "$SELF_PROXY" = "no" ]]; then
		UPSTREAM_PROXY_PORT=$(/usr/bin/iniconfig -r -f $HTTP_PNI -s "http" -o "original_server_port")
	fi

	RATING_SERVER=$(/usr/bin/iniconfig -r -f $TMUFE_CONF -s "TMUFE" -o "RatingServer")
	AU_URL=$(/usr/bin/iniconfig -r -f $INTSCAN_CONF -s "registration" -o "hostname")
	AU_SERVER=$(echo "$AU_URL" | cut -d '/' -f3)

	# ats port
	ENABLE_CACHE=`/usr/bin/iniconfig -r -f "$CACHE_CONF" -s "$CACHE_SECTION" -o "enable"`
	if [ "$ENABLE_CACHE" = "yes" ]; then
		ATS_PORT=`/usr/iwss/ats.sh get_port`
	fi

	# Debug
	#echo "MNG=$MNG"
	#echo "HTTP_PORT=$HTTP_PORT"
	#echo "FTP_PORT=$FTP_PORT"
	#echo "SSH_PORT=$SSH_PORT"
	#echo "GUEST_PORT=$GUEST_PORT"
	#echo "SERVER_FARM_PORT=$SERVER_FARM_PORT"
	#echo "POSTGRESQL_PORT=$POSTGRESQL_PORT"
	#echo "TOMCAT_PORT=$TOMCAT_PORT"
	#echo "ATS_PORT=$ATS_PORT"
	#echo "ENABLE_PING=$ENABLE_PING"
	#echo "INTERESTED_HTTP_PORTS=$INTERESTED_HTTP_PORTS"
}

#Build up the firewall including the three chains
FW_init() {
	modprobe ipv6 2> /dev/null >/dev/null

	IPTABLE -F INPUT

	# Clear and delete the custom chains
	IPTABLE -F SCAN_SERV_IN 2> /dev/null
	IPTABLE -X SCAN_SERV_IN 2> /dev/null
	IPTABLE -F ACL_IN 2> /dev/null
	IPTABLE -X ACL_IN 2> /dev/null
	IPTABLE -F LOCAL_SERV_IN 2> /dev/null
	IPTABLE -X LOCAL_SERV_IN 2> /dev/null

	# Accept established,related connections
	IPTABLE -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT # stateful

	# Create the custom chains
	IPTABLE -N SCAN_SERV_IN
	IPTABLE -N ACL_IN
	IPTABLE -N LOCAL_SERV_IN

	# Jump to custom chains
	IPTABLE -A INPUT -j SCAN_SERV_IN
	IPTABLE -A INPUT -j ACL_IN
	IPTABLE -A INPUT -j LOCAL_SERV_IN

	# Default rules
	IPTABLE -A INPUT -p tcp --dport domain -j ACCEPT
	IPTABLE -A INPUT -p udp --dport domain -j ACCEPT
	IPTABLE -A INPUT -p udp --dport bootps -j ACCEPT 	# DHCP
	IPTABLE ipv6 -A INPUT -p udp --dport 546 -j ACCEPT   # DHCPV6
	IPTABLE ipv6 -A INPUT -p tcp --dport 546 -j ACCEPT   # DHCPV6
	IPTABLE ipv6 -A INPUT -p udp --dport 547 -j ACCEPT   # DHCPV6
	IPTABLE ipv6 -A INPUT -p tcp --dport 547 -j ACCEPT   # DHCPV6
	IPTABLE -A INPUT -p udp --sport 137 -j ACCEPT  	# WINS resolve
	IPTABLE ipv4 -A INPUT -i lo -p icmp --icmp-type echo-request -j DROP
	IPTABLE ipv6 -A INPUT -i lo -p icmpv6 --icmpv6-type echo-request -j DROP
	IPTABLE -A INPUT -i lo -j ACCEPT

	# Hardening system
	IPTABLE -P INPUT DROP
	IPTABLE -A INPUT -p tcp --tcp-flags ALL SYN,FIN,PSH,URG -j DROP
	IPTABLE -A INPUT -p tcp --tcp-flags ALL FIN,PSH,URG -j DROP
	IPTABLE -A INPUT -p tcp --tcp-flags ALL NONE -j DROP
	IPTABLE -A INPUT -p tcp --tcp-flags ALL SYN,FIN -j DROP

	# HaProxy VRRP protocol
	IPTABLE -I INPUT  -p vrrp -j ACCEPT
	IPTABLE -I OUTPUT  -p vrrp -j ACCEPT
}

FW_NAT_OUT_ATS ()
{
	# content cache is only supported in forward proxy
	# so the iptable rule only apply in forward proxy mode

	local READ_INI="/usr/bin/iniconfig -r -f "
	local INTSCAN_INI="/etc/iscan/intscan.ini"
	local HTTPPROXY_PNI="/etc/iscan/IWSSPIProtocolHttpProxy.pni"

	local bridge=`$READ_INI $INTSCAN_INI -s bridge -o bridgemode`
	local transparency=`$READ_INI $HTTPPROXY_PNI -s http -o transparency`
	local self_proxy=`$READ_INI $HTTPPROXY_PNI -s http -o self_proxy`
	local icap_handler=`$READ_INI $INTSCAN_INI -s main -o protocol_config_path|grep "IWSSPIProtocolIcap"`

	#TT225749, modified by johnson, to support content cache in WCCP mode, 2011-08-24
	#[ "$bridge" -eq 0 -a "$cache_ok" == "yes" -a "$self_proxy" != "reverse" -a -z "$icap_handler" ] || return
	[[ "$bridge" -eq 0 && ( "$transparency" == "no" || "$transparency" == "wccp" ) && "$self_proxy" != "reverse" && -z "$icap_handler" ]] || return

	if [[ "$ENABLE_CACHE" = "yes" && "$ATS_PORT" != "" && "$ATS_PORT" != "0" ]]; then
		IPTABLE ipv4 -F -t nat

		if [[ "$SELF_PROXY" = "no" && "$UPSTREAM_PROXY_PORT" != "" ]]; then
			IPTABLE ipv4 -t nat -A OUTPUT -p tcp -m owner --uid-owner iscan --dport $UPSTREAM_PROXY_PORT -j REDIRECT --to-port $ATS_PORT
		else
			IPTABLE ipv4 -t nat -A OUTPUT -p tcp -m owner --uid-owner iscan --dport $WEB_SERVER_PORT -j REDIRECT --to-port $ATS_PORT
		fi
	fi
}

FW_SCAN_SERV_IN_ATS ()
{
	local ret=1
	return $ret
}

# Accept traffic to be scanned
FW_SCAN_SERV_IN()
{
	IPTABLE -F SCAN_SERV_IN

	if [ $BRIDGEMODE -eq 1 ] ; then
		IPTABLE -A SCAN_SERV_IN -p tcp -m mark --mark 0x01 -j ACCEPT # traffic redirected are marked with 0x01
	else
		FW_SCAN_SERV_IN_ATS

		for port in $HTTP_PORT; do
			for inf in $DATA_IF; do
				IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $port -j ACCEPT
			done
		done

		for inf in $DATA_IF; do
			for port in $FTP_PORT; do
				IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $port -j ACCEPT;
			done

			if [ "$((SMTP_PORT))" -ne 0 ]; then
				IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $SMTP_PORT -j ACCEPT
			fi

			if [ "$((POP3_PORT))" -ne 0 ]; then
				IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $POP3_PORT -j ACCEPT
			fi

			if [ "$((SECONDARY_PORT))" -ne 0 ]; then
				IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $SECONDARY_PORT -j ACCEPT
			fi
		done
	fi

	if [ "$((SERVER_FARM_PORT))" -ne 0 ]; then
		IPTABLE -A SCAN_SERV_IN -m state --state NEW -m tcp -p tcp --dport $SERVER_FARM_PORT -j ACCEPT
	fi

	#if [ "$((Reverse_Proxy_SSL_PORT))" -ne 0 -a "$Self_Proxy_Mode" == "reverse" ]; then
	#  IPTABLE -A SCAN_SERV_IN -m state --state NEW -m tcp -p tcp --dport $Reverse_Proxy_SSL_PORT -j ACCEPT
	#fi

	if [ ! -z "$Reverse_Proxy_SSL_PORT" ] && [ $Self_Proxy_Mode == "reverse" ]; then
		for port in $Reverse_Proxy_SSL_PORT; do
			if [ $port -ne 0 ]; then
				IPTABLE -A SCAN_SERV_IN -m state --state NEW -m tcp -p tcp --dport $port -j ACCEPT
			fi
		done
	fi

	# SSL/TLS
	for inf in $DATA_IF; do
		IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport 563 -j ACCEPT

		for port in $SSL_PORT; do
			IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $port -j ACCEPT
		done

		if [ "$GUEST_PORT" -gt 0 ] ; then
			IPTABLE -A SCAN_SERV_IN -i $inf -m state --state NEW -m tcp -p tcp --dport $GUEST_PORT -j ACCEPT
		fi
	done
}

FW_LOCAL_SERV_IN_ATS ()
{
	if [ "$ATS_PORT" = "0" -o "$ATS_PORT" = "" ] ; then
		return 1;
	fi

	if [[ "$ENABLE_CACHE" = "yes" ]]; then
		# only access for iwssd
		IPTABLE ipv4 -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --source 127.0.0.1/32 --dport $ATS_PORT -j ACCEPT
	fi

	return 1
}

# Jim-J-Wang: enable NTP traffic in cluster mode
FW_LOCAL_SERV_IN_NTP ()
{
	if [ $NTP_PORT -eq 0 ] ; then
		return 1;
	fi

	if [ -f "/etc/iscan/cluster.ini" -a "x$(/usr/bin/rwconfig -r -d '=' -f /etc/iscan/cluster.ini -s 'cluster_id')" != "x" ] ; then
		local haport=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/node.ini -s 'heartbeat_port'`
		if [ ! -z $haport ] ; then
			IPTABLE -A LOCAL_SERV_IN -i $haport -p udp --dport $NTP_PORT -j ACCEPT
			return 0
		fi
	fi
	return 1
}

FW_LOCAL_SERV_IN_CommonLog ()
{
	if [ "x$(/usr/bin/rwconfig -r -d '=' -f /etc/iscan/intscan.ini -s 'ServerRole')" == "x1" ]; then
		local logport=`/usr/bin/iniconfig -r -f /usr/trend/commonlog/config/CommonLog.ini -s 'service' -o 'srv_port'`
		if [ ! -z $logport ] ; then
			IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $logport -j ACCEPT
		fi
	fi

	return 0
}

FW_LOCAL_SERV_IN_HA()
{
#    HA_SYNC_REQ_PORT=5567
#    HA_SYNC_POST_PORT=5568

	if [ -f "/etc/iscan/cluster.ini" -a "x$(/usr/bin/rwconfig -r -d '=' -f /etc/iscan/cluster.ini -s 'cluster_id')" != "x" ] ; then
		local haport=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/node.ini -s 'heartbeat_port'`
		if [ ! -z $haport ] ; then
			IPTABLE -I OUTPUT -o $haport -j ACCEPT
			IPTABLE -I INPUT -i $haport -j ACCEPT
			return 0
		fi
	fi
	return 1
}

FW_LOCAL_SERV_IN_PAC ()
{
	local enabled=`/usr/bin/iniconfig -r -f /etc/iscan/intscan.ini -s 'pac' -o 'enable'`
	if [ "x$enabled" == "x1" ]; then
		local pacport=`/usr/bin/iniconfig -r -f /etc/iscan/intscan.ini -s 'pac' -o 'pac_port'`
		if [ ! -z $pacport ]; then
			IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $pacport -j ACCEPT
		fi
	fi
}


FW_LOCAL_SERV_IN() {
	IPTABLE -F LOCAL_SERV_IN

	# tomcat (HTTP/HTTPS)
		enable_ACL=`/usr/bin/iniconfig -r -f $NETWORK_CONF -s 'network' -o 'enable_admin_acl'`
		if [ "$enable_ACL" = "yes" ]; then
			IPTABLE ipv4 -A LOCAL_SERV_IN -i lo -p tcp ! -s 127.0.0.1 ! -d 127.0.0.1 --dport $TOMCAT_PORT -j DROP
			IPTABLE ipv6 -A LOCAL_SERV_IN -i lo -p tcp ! -s ::1 ! -d ::1 --dport $TOMCAT_PORT -j DROP
		fi

	IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $TOMCAT_PORT -j ACCEPT
	IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $SCAN_BEFORE_DELIVER_PORT -j ACCEPT

	# SSH
	if [ $SSH_PORT -gt 0 ] ; then
		IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $SSH_PORT -j ACCEPT
	fi

	# ATS
	FW_LOCAL_SERV_IN_ATS

	# SNMP
	for inf in $MGMT_IF; do
		IPTABLE -A LOCAL_SERV_IN -i $inf -p tcp --dport $SNMP_PORT -j ACCEPT		# SNMP
		IPTABLE -A LOCAL_SERV_IN -i $inf -p udp --dport $SNMP_PORT -j ACCEPT		# SNMP
		IPTABLE -A LOCAL_SERV_IN -i $inf -p tcp --dport 162 -j ACCEPT		# SNMP Trap
		IPTABLE -A LOCAL_SERV_IN -i $inf -p udp --dport 162 -j ACCEPT		# SNMP Trap
	done

	IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m udp -p udp --dport $SNMP_PORT -j ACCEPT
	IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m udp -p udp --dport 162 -j ACCEPT

	# PING
	for inf in $DATA_IF $DATA_IF_EX; do
		IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type destination-unreachable -j ACCEPT
		IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type time-exceeded -j ACCEPT
		IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-reply -j ACCEPT

		#IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type destination-unreachable -j ACCEPT
		#IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type time-exceeded -j ACCEPT
		#IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-reply -j ACCEPT
	done

	#receive NDP packets for IPv6
	#133 router-solicitation
	#134 router-advertisement
	#135 neighbour-solicitation
	#136 neighbour-advertisement
	#137 redirect
	for icmptype in 133 134 135 136 137; do
		IPTABLE ipv6 -D LOCAL_SERV_IN  -p icmpv6 --icmpv6-type $icmptype -j ACCEPT  >/dev/null 2>&1
		IPTABLE ipv6 -A LOCAL_SERV_IN  -p icmpv6 --icmpv6-type $icmptype -j ACCEPT
	done

	#for handling any error condition for NDP
	#1 destination-unreachable
	#2 packet-too-big
	#3 time-exceeded
	#4 parameter-problem
	for icmptype in 1 2 3 4; do
		IPTABLE ipv6 -D LOCAL_SERV_IN  -p icmpv6 --icmpv6-type $icmptype -j ACCEPT  >/dev/null 2>&1
		IPTABLE ipv6 -A LOCAL_SERV_IN  -p icmpv6 --icmpv6-type $icmptype -j ACCEPT
	done

	for inf in $DATA_IF; do
		if [ "$ENABLE_PING" = "yes" ] ; then
			#IPTABLE -D LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT > /dev/null 2>&1
			IPTABLE ipv4 -D LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT > /dev/null 2>&1
			IPTABLE ipv6 -D LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT > /dev/null 2>&1

			IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT

			IPTABLE ipv6 -D LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT > /dev/null 2>&1
			IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT
		else
			IPTABLE ipv4 -D LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT > /dev/null 2>&1
			IPTABLE ipv6 -D LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT > /dev/null 2>&1
		fi
	done

#	echo "magt_enable="$SEPARATED_MGMT
#	echo "mgmt_port="$MGMT_IF

	if [ "$SEPARATED_MGMT" = "yes" ] ; then
		for inf in $MGMT_IF; do
			IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type destination-unreachable -j ACCEPT
			IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type time-exceeded -j ACCEPT
			IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-reply -j ACCEPT

			IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type destination-unreachable -j ACCEPT
			IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type time-exceeded -j ACCEPT
			IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-reply -j ACCEPT

			if [ "$ENABLE_PING_MGMT" = "yes" ] ; then
				IPTABLE ipv4 -D LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT > /dev/null 2>&1
				IPTABLE ipv4 -A LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT

				IPTABLE ipv6 -D LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT > /dev/null 2>&1
				IPTABLE ipv6 -A LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT
			else
				IPTABLE ipv4 -D LOCAL_SERV_IN -i $inf -p icmp --icmp-type echo-request -j ACCEPT > /dev/null 2>&1
				IPTABLE ipv6 -D LOCAL_SERV_IN -i $inf -p icmpv6 --icmpv6-type echo-request -j ACCEPT > /dev/null 2>&1
			fi
		done
	fi

	#IPTABLE -A LOCAL_SERV_IN -i lo -p icmp --icmp-type echo-request -j ACCEPT
	IPTABLE ipv4 -A LOCAL_SERV_IN -i lo -p icmp --icmp-type echo-request -j ACCEPT
	IPTABLE ipv6 -A LOCAL_SERV_IN -i lo -p icmpv6 --icmpv6-type echo-request -j ACCEPT

	# Postgresql
	if [ $POSTGRESQL_PORT -gt 0 ] ; then
		IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $POSTGRESQL_PORT -j ACCEPT
	fi

	# SOCKS5 proxy
	if [ $SOCKS5_PROXY_PORT -gt 0 ] ; then
		IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $SOCKS5_PROXY_PORT -j ACCEPT
	fi

	# AUTH port
	is_bridge_mode=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/intscan.ini -s 'bridgemode'`
	proxy_mode=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/IWSSPIProtocolHttpProxy.pni -s 'transparency'`
	identify_method=`sudo -u iscan /usr/bin/rwconfig -r -d '=' -f /etc/iscan/intscan.ini -s 'user_identification_solution'`
	cap_enable=`sudo -u iscan /usr/bin/rwconfig -r -d '=' -f /etc/iscan/intscan.ini -s 'captive_portal'`
	cap_port=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/intscan.ini -s 'captive_portal_port'`

	if [ XX$is_bridge_mode = "XX1" -o XX$proxy_mode = "XXwccp" -o XX$proxy_mode = "XXsimple" ] && [ XX$identify_method = "XX3" ] ; then
		auth_port=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/IWSSPIProtocolHttpProxy.pni -s 'authport'`

		if [ $cap_enable = "yes" ] ; then
			IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $cap_port -j ACCEPT
		else
			IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $auth_port -j ACCEPT
		fi
	else
		if  [ XX$identify_method = "XX3" ] &&  [ ${cap_enable} = "yes" ]  ; then
			IPTABLE -A LOCAL_SERV_IN -m state --state NEW -m tcp -p tcp --dport $cap_port -j ACCEPT
		fi
	fi

	#NTP Server
	FW_LOCAL_SERV_IN_NTP

	# HA Agent
	FW_LOCAL_SERV_IN_HA

	# CommonLog
	FW_LOCAL_SERV_IN_CommonLog

	# Pac
	FW_LOCAL_SERV_IN_PAC

	return 0
}

RETURN_from_ACL_ip()
{
	RET=0
	IPTABLE -A ACL_IN -i lo -j RETURN

	F_ini_get $NETWORK_CONF 'network' '^admin_acl_file' > /dev/null
	if [ "$?" = "0" ] && [ -e $_R ] ; then
		ACL_FILE=$_R

		awk --re-interval '
			/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/ ||
			/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/([1-9]|[1-2][0-9]|3[0-2])$/ ||
			/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/ {
				cmd = sprintf("iptables -A ACL_IN -s %s -j RETURN", $0)
				if( system(cmd) != 0 ) {
					printf("Unable to to apply IPTABLE rule for %s\n", $0)
					RET=1
				}
			}
			/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\-[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/ {
				cmd = sprintf("iptables -A ACL_IN -m iprange --src-range %s -j RETURN > /dev/null 2>&1", $0)
				if( system(cmd) != 0 ) {
					printf("Unable to apply IPTABLE rule for %s\n", $0)
					RET=1
				}
			}

			/^[0-9A-Fa-f:]{1,46}$/ ||
			/^[0-9A-Fa-f:]{1,46}\/[0-9]{1,3}$/ {
				cmd = sprintf("ip6tables -A ACL_IN -s %s -j RETURN", $0)
				if( system(cmd) != 0 ) {
					printf("Unable to to apply IPTABLE rule for %s\n", $0)
					RET=1
				}
			}
			/^[0-9A-Fa-f:]{1,46}\-[0-9A-Fa-f:]{1,46}$/ {
				cmd = sprintf("ip6tables -A ACL_IN -m iprange --src-range %s -j RETURN > /dev/null 2>&1", $0)
				if( system(cmd) != 0 ) {
					printf("Unable to apply IPTABLE rule for %s\n", $0)
					RET=1
				}
		}
			' $ACL_FILE
	else
		RET=1
	fi
	return $RET
}

FW_ACL_IN() {
	IPTABLE -F ACL_IN

	F_ini_get $NETWORK_CONF 'network' '^enable_admin_acl' > /dev/null

	if [ "$?" = "0" ] && [ $_R = no ] ; then
		IPTABLE -A ACL_IN -j RETURN
		return
	fi

	F_ini_get $NETWORK_CONF 'network' '^admin_acl_file' > /dev/null
	if [ "$?" = "0" ] && [ ! -s $_R ] ; then
			IPTABLE -A ACL_IN -j RETURN
			return
	fi

	RETURN_from_ACL_ip

	# tomcat (HTTP/HTTPS)
	IPTABLE -A ACL_IN -m state --state NEW -m tcp -p tcp --dport $TOMCAT_PORT -j DROP

	# SSH
	if [ $SSH_PORT -gt 0 ] ; then
		IPTABLE -A ACL_IN -m state --state NEW -m tcp -p tcp --dport $SSH_PORT -j DROP
	fi

	IPTABLE ipv4 -A ACL_IN -p icmp --icmp-type echo-request -j DROP
	IPTABLE ipv6 -A ACL_IN -p icmpv6 --icmpv6-type echo-request -j DROP
}

FW_CUSTOM_CONF() {
	if [ -f $CUSTOM_CONF ]; then
		/bin/sh $CUSTOM_CONF
	fi
}

start_all() {
	#echo "Start IWSVA firewall rules..."
	FW_init

	FW_SCAN_SERV_IN
	FW_ACL_IN
	FW_LOCAL_SERV_IN

	FW_NAT_OUT_ATS
	FW_CUSTOM_CONF

	load_ip_conntrack_ftp
	#echo "...ok"

	return 0
}

install_fwhook() {
	if [ -L $FW_ORG ] ; then
		LNNAME=`readlink $FW_ORG`
		if [ "$LNNAME" = "$IWSVAFW" ] ; then
			echo "IWSVA firewall already been hooked"
			return 1;
		fi
	fi

	if [ -f $FW_ORG ] ; then
		/bin/cp -af $FW_ORG $FW_SYS
	else
		echo "Missing file: $FW_ORG"
	fi

	ln -sf $IWSVAFW $FW_ORG

	if [ -L $FW_ORG_V6 ] ; then
		LNNAME=`readlink $FW_ORG_V6`
		if [ "$LNNAME" = "$IWSVAFW" ] ; then
			echo "IWSVA firewall already been hooked"
			return 1
		fi
	fi

	if [ -f $FW_ORG_V6 ] ; then
		/bin/cp -af $FW_ORG_V6 $FW_SYS_V6
	else
		echo "Missing file: $FW_ORG_V6"
	fi

	ln -sf $IWSVAFW $FW_ORG_V6

	echo "IWSVA firewall hooked"

	return 0;
}

uninstall_fwhook() {
	if [ -L $FW_ORG ] ; then
		LNNAME=`readlink $FW_ORG`
		if [ "$LNNAME" != "$IWSVAFW" ] ; then
			echo "IWSVA firewall was not hooked"
			return 1
		fi
	fi

	if [ -f $FW_SYS ] ; then
		/bin/rm -f $FW_ORG
		/bin/cp -af $FW_SYS $FW_ORG
		/bin/rm -f $FW_SYS
		echo "IWSVA ipv4 hook removed"
	else
		echo "Missing file: $FW_SYS"
	fi

	if [ -L $FW_ORG_V6 ] ; then
		LNNAME=`readlink $FW_ORG_V6`
		if [ "$LNNAME" != "$IWSVAFW" ] ; then
			echo "IWSVA firewall was not hooked"
			return 1
		fi
	fi

	if [ -f $FW_SYS_V6 ] ; then
		/bin/rm -f $FW_ORG_V6
		/bin/cp -af $FW_SYS_V6 $FW_ORG_V6
		/bin/rm -f $FW_SYS_V6
		echo "IWSVA ipv6 hook removed"
	else
		echo "Missing file: $FW_SYS_V6"
	fi

	return 0
}

set_kernel_params()
{
	HASHSIZE=104856

	F_ini_get $NETWORK_CONF 'network' '^conntrack_hashsize' > /dev/null
	if [ "$?" = "0" ] && [ $_R -gt 0 ] ; then
		HASHSIZE=$_R
	fi

	if [ -e /sys/module/ip_conntrack/parameters/hashsize ] ; then
		echo $HASHSIZE > /sys/module/ip_conntrack/parameters/hashsize 2> /dev/null
	fi
	if [ -e /sys/module/nf_conntrack/parameters/hashsize ] ; then
		echo $HASHSIZE > /sys/module/nf_conntrack/parameters/hashsize 2> /dev/null
	fi
	sysctl -p > /dev/null 2>&1
}

set_fd_value ()
{
	fd_name="$2"
	fd_value="$(($3))"

	case $fd_name in
	http)
	HTTP_PORT="$fd_value"
	;;
	guest)
	GUEST_PORT="$fd_value"
	;;
	serverfarm)
	SERVER_FARM_PORT="$fd_value"
	;;
	tomcat)
	TOMCAT_PORT="$fd_value"
	;;
	esac
	return 0
}

########################### Main #######

fw_main()
{
	action=$1

	read_config
	set_fd_value $@

	case $action in
		scanning_service)
			FW_SCAN_SERV_IN
			RET=$?
			;;
		access_control_list)
			FW_ACL_IN
			RET=$?
			;;
		local_service)
			FW_ACL_IN
			RETA=$?
			FW_LOCAL_SERV_IN
			RET=$(($? | $RETA))
			;;
		service_all)
			FW_SCAN_SERV_IN
			RET0=$?
			FW_ACL_IN
			RETA=$?
			FW_LOCAL_SERV_IN
			RET=$(($? | $RET0 | $RETA))
			;;
		init_install)
			install_fwhook
		  # IWSx-02248: Error message in install.log
			exit $?
			;;
		init_uninstall)
			uninstall_fwhook
		  # IWSx-02248: Error message in install.log
			exit $?
			;;
		start)
			lsmod | grep bypass_ftphelper > /dev/null
			if [ $? -eq 0 ]; then
				rmmod bypass_ftphelper
			fi

			if [ -f $FW_SYS ]; then
				$FW_SYS start
			fi

			if [ -f $FW_SYS_V6 ]; then
				$FW_SYS_V6 start
			fi

			start_all
			set_kernel_params
			RET=$?
			;;
		stop)
			lsmod | grep bypass_ftphelper > /dev/null
			if [ $? -eq 0 ]; then
				rmmod bypass_ftphelper
			fi

			if [ -f $FW_SYS ]; then
				$FW_SYS stop
			fi

			if [ -f $FW_SYS_V6 ]; then
				$FW_SYS_V6 stop
			fi

			RET=$?
			;;
		reload|restart)
			lsmod | grep bypass_ftphelper > /dev/null
			if [ $? -eq 0 ]; then
				rmmod bypass_ftphelper
			fi

			if [ -f $FW_SYS ]; then
				$FW_SYS restart
			fi

			if [ -f $FW_SYS_V6 ]; then
				$FW_SYS_V6 restart
			fi

			start_all
			set_kernel_params
			RET=$?
			;;
		save)
			if [ -f $FW_SYS ]; then
				$FW_SYS save
			fi

			if [ -f $FW_SYS_V6 ]; then
				$FW_SYS_V6 save
			fi

			RET=$?
			;;
		*)
		echo $"Usage: $0 {start|stop|reload|restart|save|scanning_service|access_control_list|local_service}"
		exit 1
		;;
	esac

	proxy_mode=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/IWSSPIProtocolHttpProxy.pni -s 'transparency'`
	ls /usr/iwss/wccp/wccpd > /dev/null 2>&1

	if [ "$?" = "0" -a XX$proxy_mode = "XXwccp" ] ; then
		/usr/iwss/S99ISWCCPd restart >/dev/null 2>&1
	#	load_ip_conntrack_ftp
	fi

	# sshd was enabled/disable, initialize/de-initialize fail2ban

	READ_INI="/usr/bin/iniconfig -r -f "
	NETWORK_INI="/etc/iscan/network.ini"
	enable_fail2ban=`$READ_INI $NETWORK_INI -s "network" -o "enable_fail2ban"`

	/usr/bin/fail2ban-client status >/dev/null 2>&1
	running=$?

	if  [ "$enable_fail2ban" != "yes" -a $running -eq 0 ]; then
		/usr/bin/fail2ban-client stop >/dev/null 2>&1

	elif [ "$enable_fail2ban" = "yes" ]; then

		if [ -e /etc/rc3.d/S55sshd ]; then

			ssh_port=`cat /etc/ssh/sshd_config | grep -E '^Port [0-9]+' -o | cut -d' ' -f2`

			if [ -n "$ssh_port" ]; then
				fail2ban_config="/etc/fail2ban/jail.conf"

				sed -i "s/\[name=SSH, port=.*/\[name=SSH, port=$ssh_port, protocol=tcp\]/" $fail2ban_config
			fi

			if [ $running -ne 0 ]; then
				/usr/bin/fail2ban-client start >/dev/null 2>&1
			else
				/usr/bin/fail2ban-client reload >/dev/null 2>&1
			fi
		else

			if [ $running -eq 0 ]; then
				/usr/bin/fail2ban-client stop >/dev/null 2>&1
			fi
		fi
	fi

	exit $RET
}

fw_main $@

