#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import logging
import logging.config
import ConfigParser

configPath  = '../config'
handlerPath = '../handlers'
packagePath = '../packages'

currentPath = os.path.dirname(os.path.abspath(__file__))
sys.path += [os.path.join(currentPath, configPath)]
sys.path += [os.path.join(currentPath, handlerPath)]
sys.path += [os.path.join(currentPath, packagePath)]

class ReportConfig(object):
    # read port of web server from /etc/iscan/intscan.ini
    SYS_CONFIG_FILE      = os.path.join(currentPath, configPath ,"report_config.ini" )
    REPORT_SYSLOG_PATH   = os.path.join(currentPath, '../log')

    LOGGING_CONFIG_FILE  = os.path.join( currentPath, configPath ,"logging.ini" )
    REPORT_SAVE_LOCATION = '/etc/iscan/report/item/'
    REPORT_TEMP_LOCATION = os.path.join(currentPath, 'temp')
    REPORT_TEMP_FILE     = ['highcharts.js', 'jquery-1.7.1.js', 'logo.gif', 'report_template.js', 'styles.css']


    IWSVA_SYS_CONFIG_FILE   = '/etc/iscan/intscan.ini'
    REPORT_UI_PROTOCOL      = 'http'
    REPORT_UI_PORT          = '1812'
    REPORT_TEMPLATE_URL     = 'http://127.0.0.1:1812/rest/commonlog/report/schedule/template'
    REPORT_ITEM_URL         = 'http://127.0.0.1:1812/rest/commonlog/report/schedule/item'
    REPORT_ONE_TEMPLATE_URL = 'http://127.0.0.1:1812/rest/commonlog/report/template'
    REPORT_QUERY_URL        = 'http://127.0.0.1:1812/rest/commonlog/schedule_report_query'

    FAVORITE_QUERY_URL      = 'http://127.0.0.1:1812/rest/commonlog/log_favorite'
    LOG_QUERY_URL           = 'http://127.0.0.1:1812/rest/commonlog/log_page_query'

    SERVER_ROLE             = 0
    EMAIL_SERVER_IP         = 'localhost'
    EMAIL_SERVER_PORT       = '25'
    EMAIL_FROM_ADDR         = ''
    EMAIL_ENABLE            = False

    VERBOSE                 = 0

    PDF_LOG_EXPORT_LIMIT    = 1000
    CLF_TIMEOUT = 300
    FULL_REPORT = 'no'

    __inited = False
    @staticmethod
    def init():
        if ReportConfig.__inited:
            return

        '''Load agent configuration from config file.'''
        try:
            if not os.path.exists(ReportConfig.REPORT_SYSLOG_PATH):
                os.makedirs(ReportConfig.REPORT_SYSLOG_PATH)
            if not os.path.exists(ReportConfig.REPORT_SAVE_LOCATION):
                os.makedirs(ReportConfig.REPORT_SAVE_LOCATION)
            if not os.path.exists(ReportConfig.REPORT_TEMP_LOCATION):
                os.makedirs(ReportConfig.REPORT_TEMP_LOCATION)
            for filename in ReportConfig.REPORT_TEMP_FILE:
                destname = os.path.join(ReportConfig.REPORT_TEMP_LOCATION, filename)
                if not os.path.exists(destname):
                    srcname = os.path.join(currentPath, 'html', filename)
                    cmd = 'cp %s %s' % (srcname, destname)
                    os.system(cmd)


            configReader = ConfigParser.ConfigParser()
            configReader.read(ReportConfig.IWSVA_SYS_CONFIG_FILE)

            ReportConfig.REPORT_UI_PROTOCOL = configReader.get('http', 'iscan_web_protocol')
            ReportConfig.REPORT_UI_PORT     = (configReader.get('http', 'iscan_web_server')).split(':')[-1]

            ReportConfig.REPORT_TEMPLATE_URL     = ReportConfig.REPORT_TEMPLATE_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.REPORT_TEMPLATE_URL     = ReportConfig.REPORT_TEMPLATE_URL.replace('1812', ReportConfig.REPORT_UI_PORT)
            ReportConfig.REPORT_ITEM_URL         = ReportConfig.REPORT_ITEM_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.REPORT_ITEM_URL         = ReportConfig.REPORT_ITEM_URL.replace('1812', ReportConfig.REPORT_UI_PORT)
            ReportConfig.REPORT_ONE_TEMPLATE_URL = ReportConfig.REPORT_ONE_TEMPLATE_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.REPORT_ONE_TEMPLATE_URL = ReportConfig.REPORT_ONE_TEMPLATE_URL.replace('1812', ReportConfig.REPORT_UI_PORT)
            ReportConfig.REPORT_QUERY_URL = ReportConfig.REPORT_QUERY_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.REPORT_QUERY_URL = ReportConfig.REPORT_QUERY_URL.replace('1812', ReportConfig.REPORT_UI_PORT)

            ReportConfig.FAVORITE_QUERY_URL = ReportConfig.FAVORITE_QUERY_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.FAVORITE_QUERY_URL = ReportConfig.FAVORITE_QUERY_URL.replace('1812', ReportConfig.REPORT_UI_PORT)
            ReportConfig.LOG_QUERY_URL = ReportConfig.LOG_QUERY_URL.replace('http', ReportConfig.REPORT_UI_PROTOCOL)
            ReportConfig.LOG_QUERY_URL = ReportConfig.LOG_QUERY_URL.replace('1812', ReportConfig.REPORT_UI_PORT)

            ReportConfig.SERVER_ROLE = configReader.getint('CommonLog', 'ServerRole')
            ReportConfig.EMAIL_SERVER_IP   = configReader.get('notification', 'server')
            ReportConfig.EMAIL_SERVER_PORT = configReader.getint('notification', 'port')
            ReportConfig.EMAIL_FROM_ADDR   = configReader.get('notification', 'from_addr')

            ReportConfig.CLF_TIMEOUT = configReader.getint('CommonLog', 'clf_timeout') / 1000
            
            logging.config.fileConfig(ReportConfig.LOGGING_CONFIG_FILE)
            configReader.read(ReportConfig.SYS_CONFIG_FILE)
            ReportConfig.VERBOSE = configReader.getint('report', 'verbose')
            if ReportConfig.VERBOSE == 1:
                logging.getLogger().setLevel(logging.DEBUG)

            ReportConfig.PDF_LOG_EXPORT_LIMIT = configReader.getint('report', 'pdf_log_export_limit')
            ReportConfig.FULL_REPORT = configReader.get('report', 'full_report')
            

            logging.debug('Detail information is as bellow:')
            logging.debug('Verbose:             %s', ReportConfig.VERBOSE)
            logging.debug('Email enable:        %s', ReportConfig.EMAIL_ENABLE)
            logging.debug('Email server ip:     %s', ReportConfig.EMAIL_SERVER_IP)
            logging.debug('Email server port:   %s', ReportConfig.EMAIL_SERVER_PORT)
            logging.debug('Email from address:  %s', ReportConfig.EMAIL_FROM_ADDR)
            logging.debug('Report template URL: %s', ReportConfig.REPORT_TEMPLATE_URL)
            logging.debug('Report item URL:     %s', ReportConfig.REPORT_ITEM_URL)
            logging.debug('Report one template: %s', ReportConfig.REPORT_ONE_TEMPLATE_URL)
            logging.debug('IWSVA system file:   %s', ReportConfig.IWSVA_SYS_CONFIG_FILE)
            logging.debug('Report system file:  %s', ReportConfig.SYS_CONFIG_FILE)
            logging.debug('Logging config file: %s', ReportConfig.LOGGING_CONFIG_FILE)
            logging.debug('Report query url:    %s', ReportConfig.REPORT_QUERY_URL)

            ReportConfig.__inited = True
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
            logging.info('ReportConfig.__init__: %s', e)
            sys.exit(-1)
