#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import time
import urllib2
import json
import os
import shutil
from report_config import ReportConfig

from reportTemplateDef import LOG_CATEGORY_MAP


class ReportEngine(object):
    @staticmethod
    def saveReportFile(fileName):
        logging.debug("ReportEngine.saveReportFile:")
        result = None
        try:
            if not os.path.exists(ReportConfig.REPORT_SAVE_LOCATION):
                os.mkdir(ReportConfig.REPORT_SAVE_LOCATION)
            if os.path.exists(fileName):
                baseFileName = os.path.basename(fileName)
                newFileName  = os.path.join(ReportConfig.REPORT_SAVE_LOCATION, baseFileName)
                logging.debug("save (%s) to (%s)" % (fileName, newFileName))
                shutil.move(fileName, newFileName)
                result = newFileName
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.debug('ReportEngine.saveReportFile, error: %s', e)

        return result

    @staticmethod
    def listReportTemplate():
        logging.debug("ReportEngine.listReportTemplate:")
        templates = None
        try:
            response = urllib2.urlopen(ReportConfig.REPORT_TEMPLATE_URL)
            templates = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.listReportTemplate, error: %s", e)

        return templates

    @staticmethod
    def getReportTemplate(tid):
        logging.debug("ReportEngine.getReportTemplate:")
        template = None
        try:
            templateDetailURL = "%s?tid=%s" % (ReportConfig.REPORT_ONE_TEMPLATE_URL, tid)
            logging.debug(templateDetailURL)
            response = urllib2.urlopen(templateDetailURL)
            template = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.listReportTemplate, error: %s", e)
        return template

    @staticmethod
    def updateLastReportTime(tid, last_gen_time):
        logging.debug('ReportEngine.updateLastReportTime:')
        result = False
        try:
            data = {
                    'action': 'update_gen_time',
                    'parameters':{
                            'tid': tid,
                            'last_gen_time': last_gen_time
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_TEMPLATE_URL, json.dumps(data))
            result = True
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.updateLastReportTime: fail to update last generation time of report template.")
        return result

    @staticmethod
    def createReportItem(tid, state, tasktime):
        logging.debug('ReportEngine.createReportItem:')
        rid = None
        try:
            data = {
                    'action': 'create_item',
                    'parameters':{
                            'tid': tid,
                            'gen_time': tasktime,
                            'file': '',
                            'file_path': '',
                            'state': state
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_ITEM_URL, json.dumps(data))
            rid = response.read()
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.createReportItem, error: %s", e)
        return rid

    @staticmethod
    def updateReportState(rid, state, tid):
        logging.debug('ReportEngine.updateReportState:')
        result = False
        try:
            data = {
                    'action': 'update_item_state',
                    'parameters': {
                            'rid': rid,
                            'tid': tid,
                            'state': state
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_ITEM_URL, json.dumps(data))
            result   = True
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.updateReportState, error: %s", e)
        return result

    @staticmethod
    def updateReportItemFile(rid, htmlfile, pdffile, tid):
        logging.debug('ReportEngine.updateReportItemFile:')
        result = False
        try:
            data = {
                    'action': 'update_item_file_name',
                    'parameters': {
                            'rid' : rid,
                            'tid' : tid,
                            'html_file': htmlfile,
                            'pdf_file': pdffile
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_ITEM_URL, json.dumps(data))
            result = True
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.updateReportItemFile, error: %s", e)
        return result

    @staticmethod
    def listReportItem(tid):
        logging.debug('ReportEngine.listReportItem:')
        items = None
        try:
            url = "%s?tid=%s" % (ReportConfig.REPORT_ITEM_URL, tid)
            logging.debug(url)
            response = urllib2.urlopen(url)
            items = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.listReportItem, error: %s", e)
        return items

    @staticmethod
    def getReportData(reportName, timeBegin, timeEnd, filters, chartType, topN, LDAPGroups, ClientIPRange, scheduledTime, custom = False):
        logging.debug('ReportEngine.getReportData:')
        result = None
        try:
            if custom:
                reportName = LOG_CATEGORY_MAP[reportName]['facet']

            parameters = {
                'reportParam': {
                        'reportTemplate': reportName,
                        'cid'           : '0',
                        'filters'       : filters,
                        'timeBegin'     : timeBegin,
                        'timeEnd'       : timeEnd,
                        'chartType'     : chartType,
                        'scheduledTime' : scheduledTime
                }
            }
            if ReportConfig.FULL_REPORT == 'yes':
                parameters['reportParam']['topN'] = 10000
            elif topN > 0:
                parameters['reportParam']['topN'] = topN

            if LDAPGroups:
                parameters['reportParam']['LDAPGroups'] = LDAPGroups
            if ClientIPRange:
                parameters['reportParam']['IPRanges'] = ClientIPRange

            logging.debug("parameters: %s", parameters)
            response = urllib2.urlopen(ReportConfig.REPORT_QUERY_URL, json.dumps(parameters))
            result = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            result = []
            logging.exception("ReportGenerator.getReportData exception, error: %s", e)
        return result

    @staticmethod
    def cleanReportItem(tid, listNumber):
        logging.debug('ReportEngine.cleanReportItem:')
        result = False
        try:
            data = {
                    'action': 'clean_item',
                    'parameters': {
                            'tid' : tid,
                            'list_number': listNumber
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_ITEM_URL, json.dumps(data))
            result = True
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.cleanReportItem, error: %s", e)
        return result

    @staticmethod
    def updateReportCurrentExecTime(tid):
        logging.debug('ReportEngine.updateReportCurrentExecTime')
        result = False
        try:
            data = {
                    'action': 'update_current_exec_time',
                    'parameters': {
                            'tid' : tid
                    }
            }
            logging.debug(data)
            response = urllib2.urlopen(ReportConfig.REPORT_ITEM_URL, json.dumps(data))
            result = True
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.cleanReportItem, error: %s", e)
        return result

    @staticmethod
    def getLogFavorite(fid):
        logging.debug("ReportEngine.getLogFavorite:")
        fav = None
        try:
            favURL = "%s/%s" % (ReportConfig.FAVORITE_QUERY_URL, fid)
            response = urllib2.urlopen(favURL)
            fav = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            logging.error("ReportEngine.getLogFavorite %s, error: %s", fid, e)
        return fav

    @staticmethod
    def getRawLog(logCategory, timeBegin, timeEnd, filters, columns, scheduledTime):
        logging.debug('ReportEngine.getRawLog:')
        result = None

        try:
            parameters = {
                'query': {
                        'cid'           : '0',
                        'filters'       : filters,
                        'timeBegin'     : timeBegin,
                        'timeEnd'       : timeEnd,
                        'reportTemplate': LOG_CATEGORY_MAP[logCategory]['facet'],
                        'scheduledTime' : scheduledTime
                },
                'column': columns,
                'paging': {
                    'limit': ReportConfig.PDF_LOG_EXPORT_LIMIT,
                    'skip': 0
                },
                'translate':1
            }

            msgtype = LOG_CATEGORY_MAP[logCategory]['msgType']
            if not parameters['query']['filters'].has_key('CLF_MsgType'):
                parameters['query']['filters']['CLF_MsgType'] = msgtype

            logging.debug("filters: %r", parameters['query']['filters'])

            fl = []
            #convert filter to log query format
            for k,v in parameters['query']['filters'].iteritems():
                fl.append({'col': k, 'op': 'eq', 'value': v})

            parameters['query']['filters'] = fl
            logging.debug("parameters: %r", parameters)

            response = urllib2.urlopen(ReportConfig.LOG_QUERY_URL, json.dumps(parameters), timeout=ReportConfig.CLF_TIMEOUT)
            result = json.loads(response.read())
        except (urllib2.URLError, urllib2.HTTPError, Exception), e:
            result = []
            logging.exception("ReportGenerator.getRawLog exception, error: %s", e)
        return result

if __name__ == '__main__':
    pass
