#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import time
import logging
import calendar
from report_template  import ReportTemplate
from report_engine    import ReportEngine
from report_generator import ReportGenerator

def getOndemandTask(tid, taskTime):
    logging.info('reportTask.getOndemanTask: current tid=%s', tid)
    try:
        targetTask = None
        if tid and taskTime:
            template = ReportEngine.getReportTemplate(tid)
            if template:
                template = ReportTemplate(template)
                if template.checkExecState():
                    targetTask = task(tid, taskTime)
                    logging.info('get ondemand task: tid(%s), taskTime(%d)', tid, taskTime)
                else:
                    logging.info('fail to get ondemand task: tid(%s) cannot execute', tid)
            else:
                logging.info('no such tid=%s template', tid)
    except Exception, e:
        logging.exception('get ondemand task exception:%s', e)
    return targetTask

def getTasks():
    logging.info('reportTask.getTasks: currentUTCtimestamp=%d', time.time())
    taskList = []
    try:
        templates = ReportEngine.listReportTemplate()
        if templates:
            for item in templates:
                template = ReportTemplate(item)
                nextTime = template.isTimeToCreateReport()
                if nextTime:
                    #check whether the report is under generating
                    onbuild = False
                    reportItemList = ReportEngine.listReportItem(template.tid)

                    for rep in reportItemList:
                        if rep['gen_time'] == int(nextTime) and rep['state'] == 1:
                            logging.debug("Report [%r] is under generating. Ignore this time.", template.tid)
                            onbuild = True
                            break
                    if not onbuild:
                        taskList.append(task(template.tid, nextTime))
                        logging.info('get task: tid(%s), nextReportTime(%s)', template.tid, nextTime)
        else:
            logging.info("No template in db.")
    except Exception, e:
        logging.exception('reportTask.getTasks exception, error: %s)', e)
    return taskList

def executeTask(task):
    logging.info('reportTask.executeTask')
    try:
        logging.info('begin exec task: tid(%s), nextReportTime(%s)', task.gettid(),task.gettaskTime())
        newrid = ''

        generator = ReportGenerator(task.gettid(), task.gettaskTime())
        generator.prepareParam()
        generator.buildReport()
        generator.sendReportMail()
        if generator.report_status:
            newrid = generator.rid

        ReportEngine.updateLastReportTime(task.gettid(), task.gettaskTime())
        logging.debug('param status:%s, report result status:%s, mail sent status:%s',
                       generator.param_status, generator.report_status, generator.mail_status)
    except Exception, e:
        logging.exception('reportTask.executeTask exception, error: %s', e)
    finally:
        logging.info('end task: tid(%s), nextReportTime(%s)', task.gettid(), task.gettaskTime())
    return newrid



class task():
    def __init__(self, tid, taskTime):
        self.tid = tid
        self.taskTime = taskTime

    def gettid(self):
        return self.tid

    def gettaskTime(self):
        return self.taskTime

if __name__ == '__main__':
    pass