#!/usr/bin/env python

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

import ConfigParser

def convert_port(child_of_root):
    child_of_root.attrib = {'value': socks5_proxy_port}
    return

def convert_authchoice(child_of_root):
    ips = socks5_accept_srcip.split(';')

    for ip in ips:
        new_authchoice = ET.Element('authchoice')
        new_authchoice.attrib = {'source_addr': ip}
        new_select = ET.SubElement(new_authchoice, 'select')
        if socks5_auth_enable == 'yes':
            new_select.attrib = {'mechanism': 'chap'}
        else:
            new_select.attrib = {'mechanism': 'anonymous'}

        child_of_root.insert(0, new_authchoice)

    return

def convert_filter(child_of_root):

    for child in child_of_root:
        if child.tag == 'filter':
            break
    if not socks5_allow_srcip:
        pass
    else:
        ips = socks5_allow_srcip.split(';')
        for ip in ips:
            new_filter = ET.Element('filter')
            new_filter.attrib = {'source_addr': ip}
            new_action = ET.SubElement(new_filter, 'accept')
            child.insert(0, new_filter)

    if not socks5_reject_srcip:
        pass
    else:
        ips = socks5_reject_srcip.split(';')
        for ip in ips:
            new_filter = ET.Element('filter')
            new_filter.attrib = {'source_addr': ip}
            new_action = ET.SubElement(new_filter, 'reject')
            child.insert(0, new_filter)

    if not socks5_reject_dstport:
        return
    ports = socks5_reject_dstport.split(';')
    for port in ports:
        new_filter = ET.Element('filter')
        new_filter.attrib = {'dest_port': port}
        new_action = ET.SubElement(new_filter, 'reject')
        child.insert(0, new_filter)

    return

def add_users(root):
    users = socks5_auth_users.split(';')
    for user in users:
        items = user.split('/')
        elem = ET.Element('user')
        elem.attrib = {'user': items[0], 'password': items[1]}
        root.append(elem)
   
    return

def add_max_thread(root):
    elem = ET.Element('maxconnsperthread')
    elem.attrib = {'value': socks5_max_thread}
    root.append(elem)
   
    return

def add_max_timeout(root):
    elem = ET.Element('maxbindwait')
    elem.attrib = {'value': socks5_max_timeout}
    root.append(elem)
   
    return

config = ConfigParser.ConfigParser()
config.read("/etc/iscan/network.ini")
try:
    socks5_proxy_port = config.get('network', 'socks5_proxy_port')
    if not socks5_proxy_port:
        socks5_proxy_port = '1080'
except:
    socks5_proxy_port = '1080'

try:
    socks5_auth_users = config.get('network', 'socks5_auth_users')
except:
    socks5_auth_users = None

try:
    socks5_auth_enable = config.get('network', 'socks5_auth_enable') 
except:
    socks5_auth_enable = None

try:
    socks5_accept_srcip = config.get('network', 'socks5_accept_srcip')
    if not socks5_accept_srcip:
        socks5_accept_srcip = '127.0.0.1/32;172.16.0.0/12;192.168.0.0/16;10.0.0.0/8'
except:
    socks5_accept_srcip = '127.0.0.1/32;172.16.0.0/12;192.168.0.0/16;10.0.0.0/8'
   
try:
    socks5_allow_srcip = config.get('network', 'socks5_allow_srcip')
    if not socks5_allow_srcip:
        socks5_allow_srcip = None
except:
    socks5_allow_srcip = None

try:  
    socks5_reject_srcip = config.get('network', 'socks5_reject_srcip')
except:
    socks5_reject_srcip = None

try:  
    socks5_reject_dstport = config.get('network', 'socks5_reject_dstport')
except:
    socks5_reject_dstport = None

try:  
    socks5_max_thread = config.get('network', 'socks5_max_thread')
except:
    socks5_max_thread = '500'

try:  
    socks5_max_timeout = config.get('network', 'socks5_max_timeout')
except:
    socks5_max_timeout = '60'

try:  
    default_allow_action = config.get('network', 'default_allow_action')
except:
    default_allow_action = 'yes'

#print socks5_proxy_port
#print socks5_auth_users
#print socks5_auth_enable
#print socks5_accept_srcip
#print socks5_reject_srcip

if default_allow_action == 'yes':
    tree = ET.parse('/etc/antinat.xml.allow')
else:
    tree = ET.parse('/etc/antinat.xml.reject')
    
root = tree.getroot()

if socks5_auth_users:
    add_users(root)

#add_max_thread(root)
add_max_timeout(root)

for child_of_root in root:
    #print child_of_root.tag, child_of_root.attrib
    if child_of_root.tag == 'port':
        convert_port(child_of_root)

    elif child_of_root.tag == 'authchoice':
        convert_authchoice(child_of_root)

    elif child_of_root.tag == 'filter':
        convert_filter(child_of_root)
    else:
        continue

tree.write('/etc/antinat.xml.new')

