#!/usr/bin/python
#
# This script ensures iwssd is successfully started.
#
#
#
# Notice:
#   - This script has a condition check that preventing it against starting multiple instances.
#

import os
import os.path
import re
import sys
import time
import ConfigParser
import logging
import datetime
from logging.handlers import RotatingFileHandler



#############################################################
#                 Configuration section                     #
#############################################################

#IWSS main configuration file 
INTSCAN_INI                  = '/etc/iscan/intscan.ini'

HTTP_PROTO_CONF              = '/etc/iscan/IWSSPIProtocolHttpProxy.pni'

#the scripts log file.
LOG_FILE                     = '/etc/iscan/log/wccpd_monitor.log'

#logger instance
logger                       = logging.getLogger('wccpd_monitor')

#The clock for this script 
LOCKFILE                     = '/var/iwss/log/wccpd_monitor.lock'

#monitor WCCP daemon
WCCPD_DAEMON                 = 'wccpd'

#monitor HTTP daemon
HTTP_SCANNING_daemon         = 'iwssd'

#monitor FTP daemon
FTP_SCANNING_daemon          = 'isftpd'

SLEEP_TIME                   = 10 



#############################################################
#                    Function sections                      #
#############################################################

def main():

    init_log()
    
    #this script only works in WCCP mode.
    config_reader = ConfigParser.ConfigParser()
    config_reader.read(HTTP_PROTO_CONF)
    transparency = config_reader.get('http','transparency')
    
    if transparency ==  "wccp" : 
        logger.info('pass deployment mode check ')
    else:
        logger.info('This script only works in WCCP mode')
        sys.exit(1)

    
    while True:
       
        if is_another_instance_already_running():
            # we can quit or kill existing instance. There is a small chance that
            # existing instance is almost about to quit. So I decide kill existing one,
            # this sounds dirty, but reliable
            logger.info('Another instance is already running. Will kill running instance')
            sys.exit(1)
        
        
        scriptname = os.path.basename(__file__)
        start_time = int(time.time())
        
        logger.info('%s started now' % scriptname)
        
        if is_daemon_alive("wccpd"):
            logger.info('wccpd daemon is working now.')
            # need FTP traffic
            if not is_daemon_alive(FTP_SCANNING_daemon) and is_need_traffic (FTP_SCANNING_daemon) :
              logger.info('stop WCCPd because FTP scanning does not work while WCCPd will redirect FTP traffic')
              resultCommand = os.system("/usr/iwss/S99ISWCCPd stop")
                  
            # need HTTP traffic
            if not is_daemon_alive(HTTP_SCANNING_daemon) and is_need_traffic (HTTP_SCANNING_daemon) :
                logger.info('stop WCCPd because HTTP scanning does not work while WCCPd will redirect HTTP/HTTPS traffic')
                resultCommand = os.system("/usr/iwss/S99ISWCCPd stop")
                
            if not is_daemon_alive(FTP_SCANNING_daemon) and not is_daemon_alive(HTTP_SCANNING_daemon):
                logger.info('stop WCCPd because both FTP and HTTP scanning daemon do not work at the same time ')
                resultCommand = os.system("/usr/iwss/S99ISWCCPd stop") 
           
            logger.info('process checking EXIT')     
        else: 
            logger.info('wccpd daemon is not working now.')
            # need FTP traffic
            if is_need_traffic (FTP_SCANNING_daemon) and  is_daemon_alive(FTP_SCANNING_daemon):
               logger.info('need FTP traffic and FTP scanning daemon is ready to work')
               if is_need_traffic (HTTP_SCANNING_daemon) and  is_daemon_alive(HTTP_SCANNING_daemon):
                    logger.info('start WCCPd since it need HTTP traffic and HTTP scanning daemon is ready to work')
                    os.system("/usr/iwss/S99ISWCCPd start")
               if not is_need_traffic (HTTP_SCANNING_daemon) :
                   logger.info('start WCCPd since not need HTTP while FTP is ready')
                   os.system("/usr/iwss/S99ISWCCPd start")
            elif not is_need_traffic (FTP_SCANNING_daemon):
                logger.info('need not FTP traffic to check HTTP soon ')
                if is_need_traffic (HTTP_SCANNING_daemon) and  is_daemon_alive(HTTP_SCANNING_daemon):
                    logger.info('start WCCPd since not need FTP while HTTP is ready')
                    os.system("/usr/iwss/S99ISWCCPd start")
                
            logger.info('wccpd daemon check exit ')
            
        try:
            os.remove(LOCKFILE)
        except OSError, e:  ## if failed, report it back to the user ##
            print ("Error: %s - %s." % (e.LOCKFILE,e.strerror))
            
        time.sleep(SLEEP_TIME) 


def init_log():
    logger.setLevel(logging.DEBUG)
    
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    
    Rthandler = RotatingFileHandler(LOG_FILE, maxBytes=10*1024*1024,backupCount=5)
    Rthandler.setLevel(logging.INFO)
    Rthandler.setFormatter(formatter)
    logging.getLogger('').addHandler(Rthandler)  
 

def is_daemon_alive(cmd):

    command = "ps -ef | grep %s | grep -v grep | grep -v wccpd_monitor" % cmd
    resultCommand = os.system(command)
    if resultCommand == 0:
        logger.info('check process [%s] : it is working. ' % cmd)
        return True
    else:
        logger.info('check process [%s] : it is NOT working. '% cmd)
        return False

def is_need_traffic(proto):
       
    config_reader = ConfigParser.ConfigParser()
    config_reader.read(HTTP_PROTO_CONF)
    wccp_service = config_reader.get('http','wccp_service')
       

    #wccp_service=dynamic 86 protocol=tcp flags=src_ip_hash priority=120 ports=443
    
    ports=wccp_service.split("ports=",1)[1]
    
    logger.info('wccp settings [%s] ' % wccp_service)
    logger.info('wccp ports [%s]' %  ports)
    
    if proto == FTP_SCANNING_daemon :
    
         config_reader.read(INTSCAN_INI)
         ui_set_ftp_daemon = config_reader.get('ftp','ui_set_ftp_daemon')
    
         if ui_set_ftp_daemon == "stop":
             logger.info('ui_set_ftp_daemon is stop, so does not need ftp traffic' )
             return False 
    
         if ports.find("21") == -1:
             logger.info('need not to redirect FTP traffic' )
             return False
         else:
             logger.info('need to redirect FTP traffic')
             return True
    
    if proto == HTTP_SCANNING_daemon :
    
         config_reader.read(INTSCAN_INI)
         ui_set_http_daemon = config_reader.get('http','ui_set_http_daemon')
         
         if ui_set_http_daemon == "stop":
             logger.info('ui_set_http_daemon is stop, so does not need http traffic' )
             return False 
    
         if ports.find("80") == -1:
             logger.info('no 80 found' )
             if ports.find("443") == -1:
                 logger.info('need not to redirect HTTP traffic (no 443) ' )
                 return False
             else:
                 logger.info('need to redirect HTTP traffic (port 443)' )
                 return False 
         else:
             logger.info('need to redirect HTTP traffic (port 80)' )
             return True
       
    return False

def is_another_instance_already_running():
    if os.path.isfile(LOCKFILE) : 
        logger.info('find the lock file [%s]' % LOCKFILE ) 
        return True
    else:
        open(LOCKFILE, 'w+')
        if os.path.isfile(LOCKFILE) : 
            logger.info('creat the lock file [%s]' % LOCKFILE ) 
        return False

if __name__ == '__main__':
    main()
