#!/bin/sh

# global var:
#  APP_PATCH_ROOT=/etc/iscan
#  PATCHINFOINI=stargate_patch.ini
#  PATCHMGMINI=${INSTALL_DIR}/patch/config/patch_management.ini
#  PATCH_NUMBER=`F_ini_get $PATCHINFOINI PatchInfo Number`

# will be called before database is shutdown.
F_apply_database_changes()
{
    echo F_apply_database_changes
    #HA proxy create table
    result=`/usr/iwss/PostgreSQL/bin/psql iwss sa <<EOF
    CREATE TABLE tb_ha_proxy
    (
        "Device" character varying(512) NOT NULL,
        "Protocol" character varying(256) NOT NULL,
        "Port" character varying(256) NOT NULL,
        "Password" character varying(256) NOT NULL,
        "VirtualIP" character varying(512) NOT NULL,
        "VirtualIP6" character varying(512) NOT NULL,
        "Interface" character varying(256) NOT NULL,
        "DataIP" character varying(512) NOT NULL,
        "DataIP6" character varying(512) NOT NULL,
        CONSTRAINT pk_device1 PRIMARY KEY ("Device")
    );
    CREATE TABLE tb_ha_event
    (
        "Device" character varying(512) NOT NULL,
        "Event" character varying(1024),
        "LastUpdateTime" timestamp with time zone DEFAULT now()
    );
EOF`
    #client certificate table
    result=`/usr/iwss/PostgreSQL/bin/psql -h localhost -U sa -d iwss <<EOF
    CREATE TABLE lg_clientcertificate_error
    (
        "username"            varchar(256),
        "clientip"            varchar(256),
        "serverip"            varchar(256),
        "domain"              varchar(256),
        "timestamp"           TIMESTAMPTZ NOT NULL DEFAULT 'now'
    );
EOF`
    
}

# will be called before database is shutdown.
F_rollback_database_changes()
{
    #remove haproxy table
    echo F_rollback_database_changes
    result=`/usr/iwss/PostgreSQL/bin/psql iwss sa <<EOF
    DROP TABLE tb_ha_proxy;
    DROP TABLE tb_ha_event;
EOF`
    
    
    #remove client certificate table
    result=`/usr/iwss/PostgreSQL/bin/psql iwss sa <<EOF
    drop table lg_clientcertificate_error;
EOF`
    

}

F_binary_extra_copy()
{
    echo F_binary_extra_copy
}

F_update_extra_ini() {
    echo "Updating the ini file."
    
    INICONFIG="/usr/bin/iniconfig"
    IWSS_DATA=/var/iwss
    INTSCAN_CONF=$IWSS_DATA/intscan.ini

    #hotfix1608
    has_smtp_setting=`cat $INTSCAN_CONF | grep use_auth | wc -l`
    if [ $has_smtp_setting -eq 0 ];then
          $INICONFIG -w -f $INTSCAN_CONF -s notification -o use_ssl -n yes
          $INICONFIG -w -f $INTSCAN_CONF -s notification -o use_auth -n no
          $INICONFIG -w -f $INTSCAN_CONF -s notification -o auth_password -n ""
    fi

    client_ssl_setting=`cat $INTSCAN_CONF | grep client_ssl_method | cut -d '=' -f 2`

    if [ $client_ssl_setting="default" ];then
          $INICONFIG -w -f $INTSCAN_CONF -s https-scanning -o client_ssl_method -n "sslv3_3,sslv3_2,sslv3_1"
    fi

    server_ssl_setting=`cat $INTSCAN_CONF | grep server_ssl_method | cut -d '=' -f 2`

    if [ $server_ssl_setting="default" ];then
          $INICONFIG -w -f $INTSCAN_CONF -s https-scanning -o server_ssl_method -n "sslv3_3,sslv3_2,sslv3_1"
    fi
      
    #For SEGTT 336728
    LOG_AGENT_CONF=/usr/trend/commonlog/agent/config/log_agent.ini
    _log_reduction=` F_ini_get "$LOG_AGENT_CONF" "AGENT" "upload_max_queue" `
    if [ "${#_log_reduction}" -lt "1" ] ; then    
         F_ini_add_key  "$LOG_AGENT_CONF" "AGENT" "upload_max_queue" "60"     
    fi
    #end for SEGTT 336728
     
    #UK HTTPS
    _keep_ssl_version=` F_ini_get "$INTSCAN_CONF" "https-scanning" "keep_ssl_version" `
    if [ "${#_keep_ssl_version}" -lt "1" ] ; then    
         F_ini_add_key  "$INTSCAN_CONF" "https-scanning" "keep_ssl_version" "1"     
    fi
    #end of HTTPS

    #hotfix1622
    #For SEGTT 336728
    _enable_appd_daemon=` F_ini_get "$INTSCAN_CONF" "app-control" "enable_appd_daemon" `
    if [ "${#_enable_appd_daemon}" -lt "1" ] ; then    
        F_ini_add_key  "$INTSCAN_CONF" "app-control" "enable_appd_daemon" "yes"     
    fi
    #end for SEGTT 336728
     
    COMMONLOG_CONF=/usr/trend/commonlog/config/CommonLog.ini
    F_ini_add_key  "$COMMONLOG_CONF" "LDAP" "LDAP_group_support" "1"
    
    #Patch1 Feature
    echo "1. add Interrupted log setting"
    HTTP_CONF=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni
    F_ini_add_key  "$HTTP_CONF" "http" "record_interrupted_log" "no"
    F_ini_add_key  "$HTTP_CONF" "http" "enable_interrupted_log" "no"
    

    echo "2. add customize notification Setting"

    F_ini_append_section "$INTSCAN_CONF" "Customize_Domain_blockpage_Setting" "Customize_Domain_Num=0" 

    for(( i=0; i<20; i++ ))
    do
        F_ini_append_section "$INTSCAN_CONF" "Domain_$i" "path=/etc/iscan/html_template/domain_$i/"
        F_ini_add_key "$INTSCAN_CONF" "Domain_$i" "domain" "test.com"
        F_ini_add_key "$INTSCAN_CONF" "Domain_$i" "netbios" "TEST"
    done
    
    echo "3. change AU to https channel"
    sed -i "s/hostname=http:/hostname=https:/g"   /etc/iscan/intscan.ini
    F_ini_set "$INTSCAN_CONF" "registration" "use_ssl" "yes"
    
    echo "4. Full kerbose Setting"
    F_ini_add_key  "$INTSCAN_CONF" "user-identification" "enable_full_kerberos_feature" "no"
    F_ini_add_key  "$INTSCAN_CONF" "user-identification" "support_negotiate_auth" "no"
    
}

# will be called after binary copying
F_apply_extra_changes()
{
    echo F_apply_extra_changes

    F_binary_extra_copy
    F_update_extra_ini

    #update TMUSE engine for SEG 333247
    if [ "$(readlink  -- /usr/iwss/lib/libtmuseng.so.1.0)" == "libtmuseng.so.1.0.1013"  ] ; then 
         echo " need not upgrade libtmuseng.so.1.0.1013"     
    else
         echo " it need upgrade libtmuseng"
         /bin/su iscan -c "/bin/rm -f /usr/iwss/lib/libtmuseng.so /usr/iwss/lib/libtmuseng.so.1 /usr/iwss/lib/libtmuseng.so.1.0"
         /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1013 /usr/iwss/lib/libtmuseng.so.1.0 "
         /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1013 /usr/iwss/lib/libtmuseng.so.1   "
         /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1013 /usr/iwss/lib/libtmuseng.so     "
    fi
    
    ############socks feature#######
    echo "Applying SOCKS script.............."
    _path_socks=files/socks
    cd $_path_socks
    python apply_debug_module.py apply
    cd ../../
    echo "The SOCKS rpms is installed successfully."
    
    ############HA proxy############
    #1.install rpm
    _patch_ha=files/ha
    rpm -Uvh $_patch_ha/rpm/openssl-1.0.1e-42.el6_7.4.x86_64.rpm --force
    rpm -ivh $_patch_ha/rpm/lm_sensors-libs-3.1.1-17.el6.x86_64.rpm
    rpm -ivh $_patch_ha/rpm/net-snmp-libs-5.5-54.el6_7.1.x86_64.rpm
    rpm -ivh $_patch_ha/rpm/ipvsadm-1.26-4.el6.x86_64.rpm
    rpm -ivh $_patch_ha/rpm/keepalived-1.2.13-5.el6_6.x86_64.rpm
    
    

    #3.create dir and copy files
    mkdir /var/iwss/haproxy
    chown iscan:iscan /var/iwss/haproxy
    cp -f $_patch_ha/haproxy/* /var/iwss/haproxy
    chown iscan:iscan /var/iwss/haproxy/*
    
    #4.add config setting
    intscanConfig="/etc/iscan/intscan.ini";
    F_ini_append_section "$intscanConfig" "HaProxy" "mode=0" 
    F_ini_add_key "$intscanConfig" "HaProxy" "role" "0"

    F_ini_append_section "$intscanConfig" "HaProxyActiveStandby" "vip=192.168.1.1"
    F_ini_add_key "$intscanConfig" "HaProxyActiveStandby" "vip6" ""
    
    ###customize notification###############
    #create dir and copy files
    echo "##########Create dir and copy html template"
    _patch_notification=files/notification
    for(( i=0; i<20; i++ ))
    do
        mkdir /etc/iscan/html_template/domain_$i/
        chown iscan:iscan /etc/iscan/html_template/domain_$i/
        cp $_patch_notification/html/* /etc/iscan/html_template/domain_$i/
        chown iscan:iscan /etc/iscan/html_template/domain_$i/*
    done
    
    ###skynet crt replacement################
    su iscan -c "/usr/iwss/SkynetAgent/skynet_agent.py updown"
    
}

# will be called after binary copying
F_rollback_extra_changes()
{
    echo F_rollback_extra_changes
    IWSS_DATA=/var/iwss
    INTSCAN_CONF=$IWSS_DATA/intscan.ini

   #rollback TMUSE engine for SEG 333247
    PATCHMGMINI=/etc/iscan/patch/config/patch_management.ini
    _AppliedPatches=`F_ini_get ${PATCHMGMINI} PatchManagement AppliedPatches` 
    if [ "${#_AppliedPatches}" -gt "1"   ] ; then 
       echo " need not rollback libtmuseng.so.1.0.1013 since it is NOT introduced by current hotfix or patch." 
    else
       echo " it need rollback libtmuseng"
      /bin/su iscan -c "/bin/rm -f /usr/iwss/lib/libtmuseng.so /usr/iwss/lib/libtmuseng.so.1 /usr/iwss/lib/libtmuseng.so.1.0"
      /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1011 /usr/iwss/lib/libtmuseng.so.1.0 "
      /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1011 /usr/iwss/lib/libtmuseng.so.1   "
      /bin/su iscan -c "/bin/ln -s libtmuseng.so.1.0.1011 /usr/iwss/lib/libtmuseng.so     "
      rm -f   /usr/iwss/lib/libtmuseng.so.1.0.1013
    fi
    
    ####Haproxy rollback#############
    #1.rm dir and files
    rm -rf /var/iwss/haproxy
    rm -f /usr/iwss/sql/postgresql/ui_*Ha*.sql
    #2.remove config setting
    intscanConfig="/etc/iscan/intscan.ini";
    F_ini_delete_section "$intscanConfig" "HaProxy"
    F_ini_delete_section "$intscanConfig" "HaProxyActiveStandby"
    echo "# __FILE_END__" >> "$intscanConfig"
    
    ####Notification customize########
    #1.rm dir and files
    echo "##########rm dir and html template"
    for(( i=0; i<20; i++ ))
    do
        rm -rf /etc/iscan/html_template/domain_$i/
    done
    #2.remove config setting
    echo "##########Remove config Setting"

    intscanConfig="/etc/iscan/intscan.ini";

    F_ini_delete_section "$intscanConfig" "Customize_Domain_blockpage_Setting"

    for(( i=0; i<20; i++ ))
    do
        F_ini_delete_section "$intscanConfig" "Domain_$i"
    done
    
    ####AU to HTTP rollback###################
    echo "rollback AU to http channel"
    sed -i "s/hostname=https:/hostname=http:/g"   /etc/iscan/intscan.ini
    F_ini_set "$INTSCAN_CONF" "registration" "use_ssl" "no"
    
    ####Full kerbose setting rollback##########
    echo "rollback Full kerbose setting"
    F_ini_delete_key "$INTSCAN_CONF" "user-identification" "enable_full_kerberos_feature"
    F_ini_delete_key "$INTSCAN_CONF" "user-identification" "support_negotiate_auth"
    
    ####Interruped log setting rollback########
     echo "rollback Interruped log setting"
    HTTP_CONF=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni
    F_ini_delete_key  "$HTTP_CONF" "http" "record_interrupted_log"
    F_ini_delete_key  "$HTTP_CONF" "http" "enable_interrupted_log"
    
}
